@@ -1,4 +1,3 @@
-#define ENABLE_V17
 //#define LOG_FAX_AUDIO
 /*
  * SpanDSP - a series of DSP components for telephony
@@ -24,7 +23,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: fax.c,v 1.53 2007/04/08 08:16:17 steveu Exp $
+ * $Id: fax.c,v 1.63 2007/12/29 05:35:32 steveu Exp $
  */
 
 /*! \file */
@@ -49,7 +48,6 @@
 #if defined(LOG_FAX_AUDIO)
 #include <unistd.h>
 #endif
-
 #include <tiffio.h>
 
 #include "spandsp/telephony.h"
@@ -67,10 +65,8 @@
 #include "spandsp/v29tx.h"
 #include "spandsp/v27ter_rx.h"
 #include "spandsp/v27ter_tx.h"
-#if defined(ENABLE_V17)
 #include "spandsp/v17rx.h"
 #include "spandsp/v17tx.h"
-#endif
 #include "spandsp/t4.h"
 
 #include "spandsp/t30_fcf.h"
@@ -86,36 +82,42 @@ static void fax_send_hdlc(void *user_data, const uint8_t *msg, int len)
     s = (fax_state_t *) user_data;
     
     hdlc_tx_frame(&(s->hdlctx), msg, len);
-    s->first_tx_hdlc_frame = FALSE;
 }
 /*- End of function --------------------------------------------------------*/
 
-static int dummy_rx(void *s, const int16_t amp[], int len)
+static void hdlc_underflow_handler(void *user_data)
+{
+    t30_state_t *s;
+
+    s = (t30_state_t *) user_data;
+    t30_front_end_status(s, T30_FRONT_END_SEND_STEP_COMPLETE);
+}
+/*- End of function --------------------------------------------------------*/
+
+static int dummy_rx(void *user_data, const int16_t amp[], int len)
 {
     return 0;
 }
 /*- End of function --------------------------------------------------------*/
 
-#if defined(ENABLE_V17)
 static int early_v17_rx(void *user_data, const int16_t amp[], int len)
 {
     fax_state_t *s;
 
     s = (fax_state_t *) user_data;
-    v17_rx(&(s->v17rx), amp, len);
-    fsk_rx(&(s->v21rx), amp, len);
+    v17_rx(&(s->v17_rx), amp, len);
+    fsk_rx(&(s->v21_rx), amp, len);
     if (s->t30_state.rx_trained)
     {
         /* The fast modem has trained, so we no longer need to run the slow
            one in parallel. */
-        span_log(&s->logging, SPAN_LOG_FLOW, "Switching from V.17 + V.21 to V.17 (%.2fdBm0)\n", v17_rx_signal_power(&(s->v17rx)));
+        span_log(&s->logging, SPAN_LOG_FLOW, "Switching from V.17 + V.21 to V.17 (%.2fdBm0)\n", v17_rx_signal_power(&(s->v17_rx)));
         s->rx_handler = (span_rx_handler_t *) &v17_rx;
-        s->rx_user_data = &(s->v17rx);
+        s->rx_user_data = &(s->v17_rx);
     }
-    return len;
+    return 0;
 }
 /*- End of function --------------------------------------------------------*/
-#endif
 
 static int early_v27ter_rx(void *user_data, const int16_t amp[], int len)
 {
@@ -123,7 +125,7 @@ static int early_v27ter_rx(void *user_data, const int16_t amp[], int len)
 
     s = (fax_state_t *) user_data;
     v27ter_rx(&(s->v27ter_rx), amp, len);
-    fsk_rx(&(s->v21rx), amp, len);
+    fsk_rx(&(s->v21_rx), amp, len);
     if (s->t30_state.rx_trained)
     {
         /* The fast modem has trained, so we no longer need to run the slow
@@ -132,7 +134,7 @@ static int early_v27ter_rx(void *user_data, const int16_t amp[], int len)
         s->rx_handler = (span_rx_handler_t *) &v27ter_rx;
         s->rx_user_data = &(s->v27ter_rx);
     }
-    return len;
+    return 0;
 }
 /*- End of function --------------------------------------------------------*/
 
@@ -141,17 +143,17 @@ static int early_v29_rx(void *user_data, const int16_t amp[], int len)
     fax_state_t *s;
 
     s = (fax_state_t *) user_data;
-    v29_rx(&(s->v29rx), amp, len);
-    fsk_rx(&(s->v21rx), amp, len);
+    v29_rx(&(s->v29_rx), amp, len);
+    fsk_rx(&(s->v21_rx), amp, len);
     if (s->t30_state.rx_trained)
     {
         /* The fast modem has trained, so we no longer need to run the slow
            one in parallel. */
-        span_log(&s->logging, SPAN_LOG_FLOW, "Switching from V.29 + V.21 to V.29 (%.2fdBm0)\n", v29_rx_signal_power(&(s->v29rx)));
+        span_log(&s->logging, SPAN_LOG_FLOW, "Switching from V.29 + V.21 to V.29 (%.2fdBm0)\n", v29_rx_signal_power(&(s->v29_rx)));
         s->rx_handler = (span_rx_handler_t *) &v29_rx;
-        s->rx_user_data = &(s->v29rx);
+        s->rx_user_data = &(s->v29_rx);
     }
-    return len;
+    return 0;
 }
 /*- End of function --------------------------------------------------------*/
 
@@ -167,7 +169,7 @@ int fax_rx(fax_state_t *s, int16_t *amp, int len)
         amp[i] = dc_restore(&(s->dc_restore), amp[i]);
     s->rx_handler(s->rx_user_data, amp, len);
     t30_timer_update(&(s->t30_state), len);
-    return  0;
+    return 0;
 }
 /*- End of function --------------------------------------------------------*/
 
@@ -205,7 +207,7 @@ int fax_tx(fax_state_t *s, int16_t *amp, int max_len)
         {
             /* Allow for a change of tx handler within a block */
             if (set_next_tx_type(s)  &&  s->current_tx_type != T30_MODEM_NONE  &&  s->current_tx_type != T30_MODEM_DONE)
-                t30_send_complete(&(s->t30_state));
+                t30_front_end_status(&(s->t30_state), T30_FRONT_END_SEND_COMPLETE);
             if (!s->transmit)
             {
                 if (s->transmit_on_idle)
@@ -266,9 +268,10 @@ static void fax_set_rx_type(void *user_data, int type, int short_train, int use_
     case T30_MODEM_V21:
         if (s->flush_handler)
             s->flush_handler(s, s->flush_user_data, 3);
-        fsk_rx_init(&(s->v21rx), &preset_fsk_specs[FSK_V21CH2], TRUE, 0, (put_bit_func_t) hdlc_rx_put_bit, put_bit_user_data);
+        fsk_rx_init(&(s->v21_rx), &preset_fsk_specs[FSK_V21CH2], TRUE, (put_bit_func_t) hdlc_rx_put_bit, put_bit_user_data);
+        fsk_rx_signal_cutoff(&(s->v21_rx), -45.5);
         s->rx_handler = (span_rx_handler_t *) &fsk_rx;
-        s->rx_user_data = &(s->v21rx);
+        s->rx_user_data = &(s->v21_rx);
         break;
     case T30_MODEM_V27TER_2400:
         v27ter_rx_restart(&(s->v27ter_rx), 2400, FALSE);
@@ -283,43 +286,41 @@ static void fax_set_rx_type(void *user_data, int type, int short_train, int use_
         s->rx_user_data = s;
         break;
     case T30_MODEM_V29_7200:
-        v29_rx_restart(&(s->v29rx), 7200, FALSE);
-        v29_rx_set_put_bit(&(s->v29rx), put_bit_func, put_bit_user_data);
+        v29_rx_restart(&(s->v29_rx), 7200, FALSE);
+        v29_rx_set_put_bit(&(s->v29_rx), put_bit_func, put_bit_user_data);
         s->rx_handler = (span_rx_handler_t *) &early_v29_rx;
         s->rx_user_data = s;
         break;
     case T30_MODEM_V29_9600:
-        v29_rx_restart(&(s->v29rx), 9600, FALSE);
-        v29_rx_set_put_bit(&(s->v29rx), put_bit_func, put_bit_user_data);
+        v29_rx_restart(&(s->v29_rx), 9600, FALSE);
+        v29_rx_set_put_bit(&(s->v29_rx), put_bit_func, put_bit_user_data);
         s->rx_handler = (span_rx_handler_t *) &early_v29_rx;
         s->rx_user_data = s;
         break;
-#if defined(ENABLE_V17)
     case T30_MODEM_V17_7200:
-        v17_rx_restart(&(s->v17rx), 7200, short_train);
-        v17_rx_set_put_bit(&(s->v17rx), put_bit_func, put_bit_user_data);
+        v17_rx_restart(&(s->v17_rx), 7200, short_train);
+        v17_rx_set_put_bit(&(s->v17_rx), put_bit_func, put_bit_user_data);
         s->rx_handler = (span_rx_handler_t *) &early_v17_rx;
         s->rx_user_data = s;
         break;
     case T30_MODEM_V17_9600:
-        v17_rx_restart(&(s->v17rx), 9600, short_train);
-        v17_rx_set_put_bit(&(s->v17rx), put_bit_func, put_bit_user_data);
+        v17_rx_restart(&(s->v17_rx), 9600, short_train);
+        v17_rx_set_put_bit(&(s->v17_rx), put_bit_func, put_bit_user_data);
         s->rx_handler = (span_rx_handler_t *) &early_v17_rx;
         s->rx_user_data = s;
         break;
     case T30_MODEM_V17_12000:
-        v17_rx_restart(&(s->v17rx), 12000, short_train);
-        v17_rx_set_put_bit(&(s->v17rx), put_bit_func, put_bit_user_data);
+        v17_rx_restart(&(s->v17_rx), 12000, short_train);
+        v17_rx_set_put_bit(&(s->v17_rx), put_bit_func, put_bit_user_data);
         s->rx_handler = (span_rx_handler_t *) &early_v17_rx;
         s->rx_user_data = s;
         break;
     case T30_MODEM_V17_14400:
-        v17_rx_restart(&(s->v17rx), 14400, short_train);
-        v17_rx_set_put_bit(&(s->v17rx), put_bit_func, put_bit_user_data);
+        v17_rx_restart(&(s->v17_rx), 14400, short_train);
+        v17_rx_set_put_bit(&(s->v17_rx), put_bit_func, put_bit_user_data);
         s->rx_handler = (span_rx_handler_t *) &early_v17_rx;
         s->rx_user_data = s;
         break;
-#endif
     case T30_MODEM_DONE:
         span_log(&s->logging, SPAN_LOG_FLOW, "FAX exchange complete\n");
     default:
@@ -341,7 +342,6 @@ static void fax_set_tx_type(void *user_data, int type, int short_train, int use_
     span_log(&s->logging, SPAN_LOG_FLOW, "Set tx type %d\n", type);
     if (s->current_tx_type == type)
         return;
-    s->first_tx_hdlc_frame = TRUE;
     if (use_hdlc)
     {
         get_bit_func = (get_bit_func_t) hdlc_tx_get_bit;
@@ -401,9 +401,9 @@ static void fax_set_tx_type(void *user_data, int type, int short_train, int use_
         s->transmit = TRUE;
         break;
     case T30_MODEM_V21:
-        fsk_tx_init(&(s->v21tx), &preset_fsk_specs[FSK_V21CH2], get_bit_func, get_bit_user_data);
+        fsk_tx_init(&(s->v21_tx), &preset_fsk_specs[FSK_V21CH2], get_bit_func, get_bit_user_data);
         /* The spec says 1s +-15% of preamble. So, the minimum is 32 octets. */
-        hdlc_tx_preamble(&(s->hdlctx), 32);
+        hdlc_tx_flags(&(s->hdlctx), 32);
         /* Pause before switching from phase C, as per T.30 5.3.2.2. If we omit this, the receiver
            might not see the carrier fall between the high speed and low speed sections. In practice,
            a 75ms gap before any V.21 transmission is harmless, adds little to the overall length of
@@ -412,7 +412,7 @@ static void fax_set_tx_type(void *user_data, int type, int short_train, int use_
         s->tx_handler = (span_tx_handler_t *) &silence_gen;
         s->tx_user_data = &(s->silence_gen);
         s->next_tx_handler = (span_tx_handler_t *) &fsk_tx;
-        s->next_tx_user_data = &(s->v21tx);
+        s->next_tx_user_data = &(s->v21_tx);
         s->transmit = TRUE;
         break;
     case T30_MODEM_V27TER_2400:
@@ -423,7 +423,7 @@ static void fax_set_tx_type(void *user_data, int type, int short_train, int use_
         s->tx_user_data = &(s->silence_gen);
         s->next_tx_handler = (span_tx_handler_t *) &v27ter_tx;
         s->next_tx_user_data = &(s->v27ter_tx);
-        hdlc_tx_preamble(&(s->hdlctx), 60);
+        hdlc_tx_flags(&(s->hdlctx), 60);
         s->transmit = TRUE;
         break;
     case T30_MODEM_V27TER_4800:
@@ -434,77 +434,75 @@ static void fax_set_tx_type(void *user_data, int type, int short_train, int use_
         s->tx_user_data = &(s->silence_gen);
         s->next_tx_handler = (span_tx_handler_t *) &v27ter_tx;
         s->next_tx_user_data = &(s->v27ter_tx);
-        hdlc_tx_preamble(&(s->hdlctx), 120);
+        hdlc_tx_flags(&(s->hdlctx), 120);
         s->transmit = TRUE;
         break;
     case T30_MODEM_V29_7200:
         silence_gen_alter(&(s->silence_gen), ms_to_samples(75));
-        v29_tx_restart(&(s->v29tx), 7200, s->use_tep);
-        v29_tx_set_get_bit(&(s->v29tx), get_bit_func, get_bit_user_data);
+        v29_tx_restart(&(s->v29_tx), 7200, s->use_tep);
+        v29_tx_set_get_bit(&(s->v29_tx), get_bit_func, get_bit_user_data);
         s->tx_handler = (span_tx_handler_t *) &silence_gen;
         s->tx_user_data = &(s->silence_gen);
         s->next_tx_handler = (span_tx_handler_t *) &v29_tx;
-        s->next_tx_user_data = &(s->v29tx);
-        hdlc_tx_preamble(&(s->hdlctx), 180);
+        s->next_tx_user_data = &(s->v29_tx);
+        hdlc_tx_flags(&(s->hdlctx), 180);
         s->transmit = TRUE;
         break;
     case T30_MODEM_V29_9600:
         silence_gen_alter(&(s->silence_gen), ms_to_samples(75));
-        v29_tx_restart(&(s->v29tx), 9600, s->use_tep);
-        v29_tx_set_get_bit(&(s->v29tx), get_bit_func, get_bit_user_data);
+        v29_tx_restart(&(s->v29_tx), 9600, s->use_tep);
+        v29_tx_set_get_bit(&(s->v29_tx), get_bit_func, get_bit_user_data);
         s->tx_handler = (span_tx_handler_t *) &silence_gen;
         s->tx_user_data = &(s->silence_gen);
         s->next_tx_handler = (span_tx_handler_t *) &v29_tx;
-        s->next_tx_user_data = &(s->v29tx);
-        hdlc_tx_preamble(&(s->hdlctx), 240);
+        s->next_tx_user_data = &(s->v29_tx);
+        hdlc_tx_flags(&(s->hdlctx), 240);
         s->transmit = TRUE;
         break;
-#if defined(ENABLE_V17)
     case T30_MODEM_V17_7200:
         silence_gen_alter(&(s->silence_gen), ms_to_samples(75));
-        v17_tx_restart(&(s->v17tx), 7200, s->use_tep, short_train);
-        v17_tx_set_get_bit(&(s->v17tx), get_bit_func, get_bit_user_data);
+        v17_tx_restart(&(s->v17_tx), 7200, s->use_tep, short_train);
+        v17_tx_set_get_bit(&(s->v17_tx), get_bit_func, get_bit_user_data);
         s->tx_handler = (span_tx_handler_t *) &silence_gen;
         s->tx_user_data = &(s->silence_gen);
         s->next_tx_handler = (span_tx_handler_t *) &v17_tx;
-        s->next_tx_user_data = &(s->v17tx);
-        hdlc_tx_preamble(&(s->hdlctx), 180);
+        s->next_tx_user_data = &(s->v17_tx);
+        hdlc_tx_flags(&(s->hdlctx), 180);
         s->transmit = TRUE;
         break;
     case T30_MODEM_V17_9600:
         silence_gen_alter(&(s->silence_gen), ms_to_samples(75));
-        v17_tx_restart(&(s->v17tx), 9600, s->use_tep, short_train);
-        v17_tx_set_get_bit(&(s->v17tx), get_bit_func, get_bit_user_data);
+        v17_tx_restart(&(s->v17_tx), 9600, s->use_tep, short_train);
+        v17_tx_set_get_bit(&(s->v17_tx), get_bit_func, get_bit_user_data);
         s->tx_handler = (span_tx_handler_t *) &silence_gen;
         s->tx_user_data = &(s->silence_gen);
         s->next_tx_handler = (span_tx_handler_t *) &v17_tx;
-        s->next_tx_user_data = &(s->v17tx);
-        hdlc_tx_preamble(&(s->hdlctx), 240);
+        s->next_tx_user_data = &(s->v17_tx);
+        hdlc_tx_flags(&(s->hdlctx), 240);
         s->transmit = TRUE;
         break;
     case T30_MODEM_V17_12000:
         silence_gen_alter(&(s->silence_gen), ms_to_samples(75));
-        v17_tx_restart(&(s->v17tx), 12000, s->use_tep, short_train);
-        v17_tx_set_get_bit(&(s->v17tx), get_bit_func, get_bit_user_data);
+        v17_tx_restart(&(s->v17_tx), 12000, s->use_tep, short_train);
+        v17_tx_set_get_bit(&(s->v17_tx), get_bit_func, get_bit_user_data);
         s->tx_handler = (span_tx_handler_t *) &silence_gen;
         s->tx_user_data = &(s->silence_gen);
         s->next_tx_handler = (span_tx_handler_t *) &v17_tx;
-        s->next_tx_user_data = &(s->v17tx);
-        hdlc_tx_preamble(&(s->hdlctx), 300);
+        s->next_tx_user_data = &(s->v17_tx);
+        hdlc_tx_flags(&(s->hdlctx), 300);
         s->transmit = TRUE;
         break;
     case T30_MODEM_V17_14400:
         silence_gen_alter(&(s->silence_gen), ms_to_samples(75));
-        v17_tx_restart(&(s->v17tx), 14400, s->use_tep, short_train);
-        v17_tx_set_get_bit(&(s->v17tx), get_bit_func, get_bit_user_data);
+        v17_tx_restart(&(s->v17_tx), 14400, s->use_tep, short_train);
+        v17_tx_set_get_bit(&(s->v17_tx), get_bit_func, get_bit_user_data);
         s->tx_handler = (span_tx_handler_t *) &silence_gen;
         s->tx_user_data = &(s->silence_gen);
         s->next_tx_handler = (span_tx_handler_t *) &v17_tx;
-        s->next_tx_user_data = &(s->v17tx);
-        hdlc_tx_preamble(&(s->hdlctx), 360);
+        s->next_tx_user_data = &(s->v17_tx);
+        hdlc_tx_flags(&(s->hdlctx), 360);
         s->transmit = TRUE;
         break;
-#endif
     case T30_MODEM_DONE:
         span_log(&s->logging, SPAN_LOG_FLOW, "FAX exchange complete\n");
         /* Fall through */
@@ -552,23 +550,17 @@ fax_state_t *fax_init(fax_state_t *s, int calling_party)
              fax_send_hdlc,
              (void *) s);
     t30_set_supported_modems(&(s->t30_state),
-#if defined(ENABLE_V17)
-                             T30_SUPPORT_V27TER | T30_SUPPORT_V29 | T30_SUPPORT_V17);
-#else    
                              T30_SUPPORT_V27TER | T30_SUPPORT_V29);
-#endif
     hdlc_rx_init(&(s->hdlcrx), FALSE, FALSE, 5, t30_hdlc_accept, &(s->t30_state));
-    fsk_rx_init(&(s->v21rx), &preset_fsk_specs[FSK_V21CH2], TRUE, 0, (put_bit_func_t) hdlc_rx_put_bit, &(s->hdlcrx));
-    hdlc_tx_init(&(s->hdlctx), FALSE, 2, FALSE, t30_send_complete, &(s->t30_state));
-    s->first_tx_hdlc_frame = TRUE;
-    fsk_tx_init(&(s->v21tx), &preset_fsk_specs[FSK_V21CH2], (get_bit_func_t) hdlc_tx_get_bit, &(s->hdlctx));
-#if defined(ENABLE_V17)
-    v17_rx_init(&(s->v17rx), 14400, t30_non_ecm_put_bit, &(s->t30_state));
-    v17_tx_init(&(s->v17tx), 14400, s->use_tep, t30_non_ecm_get_bit, &(s->t30_state));
-#endif
-    v29_rx_init(&(s->v29rx), 9600, t30_non_ecm_put_bit, &(s->t30_state));
-    v29_rx_signal_cutoff(&(s->v29rx), -45.5);
-    v29_tx_init(&(s->v29tx), 9600, s->use_tep, t30_non_ecm_get_bit, &(s->t30_state));
+    fsk_rx_init(&(s->v21_rx), &preset_fsk_specs[FSK_V21CH2], TRUE, (put_bit_func_t) hdlc_rx_put_bit, &(s->hdlcrx));
+    fsk_rx_signal_cutoff(&(s->v21_rx), -45.5);
+    hdlc_tx_init(&(s->hdlctx), FALSE, 2, FALSE, hdlc_underflow_handler, &(s->t30_state));
+    fsk_tx_init(&(s->v21_tx), &preset_fsk_specs[FSK_V21CH2], (get_bit_func_t) hdlc_tx_get_bit, &(s->hdlctx));
+    v17_rx_init(&(s->v17_rx), 14400, t30_non_ecm_put_bit, &(s->t30_state));
+    v17_tx_init(&(s->v17_tx), 14400, s->use_tep, t30_non_ecm_get_bit, &(s->t30_state));
+    v29_rx_init(&(s->v29_rx), 9600, t30_non_ecm_put_bit, &(s->t30_state));
+    v29_rx_signal_cutoff(&(s->v29_rx), -45.5);
+    v29_tx_init(&(s->v29_tx), 9600, s->use_tep, t30_non_ecm_get_bit, &(s->t30_state));
     v27ter_rx_init(&(s->v27ter_rx), 4800, t30_non_ecm_put_bit, &(s->t30_state));
     v27ter_tx_init(&(s->v27ter_tx), 4800, s->use_tep, t30_non_ecm_get_bit, &(s->t30_state));
     silence_gen_init(&(s->silence_gen), 0);
@@ -583,7 +575,7 @@ fax_state_t *fax_init(fax_state_t *s, int calling_party)
         time(&now);
         tm = localtime(&now);
         sprintf(buf,
-                "/tmp/fax-rx-audio-%x-%02d%02d%02d%02d%02d%02d",
+                "/tmp/fax-rx-audio-%p-%02d%02d%02d%02d%02d%02d",
                 s,
                 tm->tm_year%100,
                 tm->tm_mon + 1,
@@ -593,7 +585,7 @@ fax_state_t *fax_init(fax_state_t *s, int calling_party)
                 tm->tm_sec);
         s->fax_audio_rx_log = open(buf, O_CREAT | O_TRUNC | O_WRONLY, 0666);
         sprintf(buf,
-                "/tmp/fax-tx-audio-%x-%02d%02d%02d%02d%02d%02d",
+                "/tmp/fax-tx-audio-%p-%02d%02d%02d%02d%02d%02d",
                 s,
                 tm->tm_year%100,
                 tm->tm_mon + 1,
@@ -615,6 +607,14 @@ int fax_release(fax_state_t *s)
 }
 /*- End of function --------------------------------------------------------*/
 
+int fax_free(fax_state_t *s)
+{
+    t30_release(&s->t30_state);
+    free(s);
+    return 0;
+}
+/*- End of function --------------------------------------------------------*/
+
 void fax_set_flush_handler(fax_state_t *s, fax_flush_handler_t *handler, void *user_data)
 {
     s->flush_handler = handler;
