@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: modem_echo.c,v 1.16 2006/10/24 13:45:26 steveu Exp $
+ * $Id: modem_echo.c,v 1.17 2006/11/19 14:07:24 steveu Exp $
  */
 
 /*! \file */
@@ -40,7 +40,12 @@
 #include <inttypes.h>
 #include <string.h>
 #include <stdio.h>
+#if defined(HAVE_TGMATH_H)
 #include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
+#include <math.h>
+#endif
 
 #include "spandsp/telephony.h"
 #include "spandsp/bit_operations.h"
