@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: plc.c,v 1.15 2006/10/24 13:45:26 steveu Exp $
+ * $Id: plc.c,v 1.18 2007/11/30 12:20:33 steveu Exp $
  */
 
 /*! \file */
@@ -35,7 +35,12 @@
 #include <inttypes.h>
 #include <stdlib.h>
 #include <string.h>
+#if defined(HAVE_TGMATH_H)
 #include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
+#include <math.h>
+#endif
 #include <limits.h>
 
 #include "spandsp/telephony.h"
@@ -241,9 +246,10 @@ plc_state_t *plc_init(plc_state_t *s)
 }
 /*- End of function --------------------------------------------------------*/
 
-int plc_release(plc_state_t *s)
+int plc_free(plc_state_t *s)
 {
-    free(s);
+    if (s)
+        free(s);
     return 0;
 }
 /*- End of function --------------------------------------------------------*/
