@@ -48,13 +48,13 @@
  * 2550 Garcia Avenue
  * Mountain View, California  94043
  *
- * $Id: g726.h,v 1.12 2006/10/24 13:45:28 steveu Exp $
+ * $Id: g726.h,v 1.18 2007/12/13 11:31:32 steveu Exp $
  */
 
 /*! \file */
 
-#if !defined(_G726_H_)
-#define _G726_H_
+#if !defined(_SPANDSP_G726_H_)
+#define _SPANDSP_G726_H_
 
 /*! \page g726_page G.726 encoding and decoding
 \section g726_page_sec_1 What does it do?
@@ -91,7 +91,7 @@ typedef int16_t (*g726_decoder_func_t)(struct g726_state_s *s, uint8_t code);
 
 typedef uint8_t (*g726_encoder_func_t)(struct g726_state_s *s, int16_t amp);
 
-/*
+/*!
  * The following is the definition of the state structure
  * used by the G.726 encoder and decoder to preserve their internal
  * state between successive calls.  The meanings of the majority
@@ -107,8 +107,8 @@ typedef struct g726_state_s
     /*! The external coding, for tandem operation */
     int ext_coding;
     /*! The number of bits per sample */
-    int bits_per_sample;
-    /*! One fo the G.726_PACKING_xxx options */
+    unsigned int bits_per_sample;
+    /*! One of the G.726_PACKING_xxx options */
     int packing;
 
     /*! Locked or steady state step size multiplier. */
@@ -137,14 +137,18 @@ typedef struct g726_state_s
     /*! Delayed tone detect */
     int td;
     
+    /*! \brief The bit stream processing context. */
     bitstream_state_t bs;
 
+    /*! \brief The current encoder function. */
     g726_encoder_func_t enc_func;
+    /*! \brief The current decoder function. */
     g726_decoder_func_t dec_func;
 } g726_state_t;
 
-#ifdef __cplusplus
-extern "C" {
+#if defined(__cplusplus)
+extern "C"
+{
 #endif
 
 /*! Initialise a G.726 encode or decode context.
@@ -183,7 +187,7 @@ int g726_encode(g726_state_t *s,
                 const int16_t amp[],
                 int len);
 
-#ifdef __cplusplus
+#if defined(__cplusplus)
 }
 #endif
 
