@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: gsm0610.h,v 1.10 2007/04/08 08:16:18 steveu Exp $
+ * $Id: gsm0610.h,v 1.13 2007/12/13 11:31:32 steveu Exp $
  */
 
 #if !defined(_SPANDSP_GSM0610_H_)
@@ -79,21 +79,26 @@ typedef struct
 
     int16_t dp0[280];
 
-    int16_t z1;         /* preprocessing,   Offset_com. */
-    int32_t L_z2;       /*                  Offset_com. */
-    int16_t mp;         /*                  Preemphasis */
+    /*! Preprocessing */
+    int16_t z1;
+    int32_t L_z2;
+    /*! Pre-emphasis */
+    int16_t mp;
 
-    int16_t u[8];       /* short_term_delay_filter */
+    /*! Short term delay filter */
+    int16_t u[8];
     int16_t LARpp[2][8];
     int16_t j;
 
-    int16_t nrp;        /* long term synthesis */
-    int16_t v[9];       /* short term synthesis */
-    int16_t msr;        /* decoder postprocessing */
-
-    int16_t e[50];      /* encoder */
-    uint8_t	frame_index;
-    uint8_t	frame_chain;
+    /*! Long term synthesis */
+    int16_t nrp;
+    /*! Short term synthesis */
+    int16_t v[9];
+    /*! Decoder postprocessing */
+    int16_t msr;
+    
+    /*! Encoder data */
+    int16_t e[50];
 } gsm0610_state_t;
 
 #if defined(__cplusplus)
@@ -107,13 +112,22 @@ extern "C"
     \return A pointer to the GSM 06.10 context, or NULL for error. */
 gsm0610_state_t *gsm0610_init(gsm0610_state_t *s, int packing);
 
+/*! Release a GSM 06.10 encode or decode context.
+    \param s The GSM 06.10 context
+    \return 0 for success, else -1. */
 int gsm0610_release(gsm0610_state_t *s);
 
+/*! Set the packing format for a GSM 06.10 encode or decode context.
+    \param s The GSM 06.10 context
+    \param packing One of the GSM0610_PACKING_xxx options.
+    \return 0 for success, else -1. */
+int gsm0610_set_packing(gsm0610_state_t *s, int packing);
+
 /*! Encode a buffer of linear PCM data to GSM 06.10.
     \param s The GSM 06.10 context.
-    \param ima_data The GSM 06.10 data produced.
+    \param code The GSM 06.10 data produced.
     \param amp The audio sample buffer.
-    \param len The number of samples in the buffer.
+    \param quant The number of samples in the buffer.
     \return The number of bytes of GSM 06.10 data produced. */
 int gsm0610_encode(gsm0610_state_t *s, uint8_t code[], const int16_t amp[], int quant);
 
@@ -125,16 +139,32 @@ int gsm0610_encode(gsm0610_state_t *s, uint8_t code[], const int16_t amp[], int
     \return The number of samples returned. */
 int gsm0610_decode(gsm0610_state_t *s, int16_t amp[], const uint8_t code[], int quant);
 
-int gsm0610_pack_none(uint8_t c[], gsm0610_frame_t *s);
+int gsm0610_pack_none(uint8_t c[], const gsm0610_frame_t *s);
 
-int gsm0610_pack_wav49(uint8_t c[], gsm0610_frame_t *s, int half);
+/*! Pack a pair of GSM 06.10 frames in the format used for wave files (wave type 49).
+    \param c The buffer for the packed data. This must be at least 65 bytes long.
+    \param s A pointer to the frames to be packed.
+    \return The number of bytes generated. */
+int gsm0610_pack_wav49(uint8_t c[], const gsm0610_frame_t *s);
 
-int gsm0610_pack_voip(uint8_t c[], gsm0610_frame_t *s);
+/*! Pack a GSM 06.10 frames in the format used for VoIP.
+    \param c The buffer for the packed data. This must be at least 33 bytes long.
+    \param s A pointer to the frame to be packed.
+    \return The number of bytes generated. */
+int gsm0610_pack_voip(uint8_t c[], const gsm0610_frame_t *s);
 
 int gsm0610_unpack_none(gsm0610_frame_t *s, const uint8_t c[]);
 
-int gsm0610_unpack_wav49(gsm0610_frame_t *s, const uint8_t c[], int half);
+/*! Unpack a pair of GSM 06.10 frames from the format used for wave files (wave type 49).
+    \param s A pointer to a buffer into which the frames will be packed.
+    \param c The buffer containing the data to be unpacked. This must be at least 65 bytes long.
+    \return The number of bytes absorbed. */
+int gsm0610_unpack_wav49(gsm0610_frame_t *s, const uint8_t c[]);
 
+/*! Unpack a GSM 06.10 frame from the format used for VoIP.
+    \param s A pointer to a buffer into which the frame will be packed.
+    \param c The buffer containing the data to be unpacked. This must be at least 33 bytes long.
+    \return The number of bytes absorbed. */
 int gsm0610_unpack_voip(gsm0610_frame_t *s, const uint8_t c[]);
 
 #if defined(__cplusplus)
