@@ -22,14 +22,18 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: queue.h,v 1.9 2007/04/30 13:13:36 steveu Exp $
+ * $Id: queue.h,v 1.12 2007/11/30 12:20:35 steveu Exp $
  */
 
 /*! \file */
 
 /*! \page queue_page Queuing
 \section queue_page_sec_1 What does it do?
-???.
+This module provides lock free queuing for either octet streams or messages.
+Specifically, lock free means one thread can write and another can read without
+locking the queue. It does NOT means a free-for-all is possible, with many
+threads writing or many threads reading. Those things would require locking,
+to avoid conflicts between the multiple threads acting on one end of the queue.
 
 \section queue_page_sec_2 How does it work?
 ???.
@@ -95,6 +99,12 @@ int queue_view(queue_state_t *s, uint8_t *buf, int len);
     \return the number of bytes returned. */
 int queue_read(queue_state_t *s, uint8_t *buf, int len);
 
+/*! Read a byte from a queue.
+    \brief Read a byte from a queue.
+    \param s The queue context.
+    \return the byte, or -1 if the queue is empty. */
+int queue_read_byte(queue_state_t *s);
+
 /*! Write bytes to a queue.
     \brief Write bytes to a queue.
     \param s The queue context.
@@ -103,6 +113,13 @@ int queue_read(queue_state_t *s, uint8_t *buf, int len);
     \return the number of bytes actually written. */
 int queue_write(queue_state_t *s, const uint8_t *buf, int len);
 
+/*! Write a byte to a queue.
+    \brief Write a byte to a queue.
+    \param s The queue context.
+    \param byte The byte to be written.
+    \return the number of bytes actually written. */
+int queue_write_byte(queue_state_t *s, uint8_t byte);
+
 /*! Test the length of the message at the head of a queue.
     \brief Test message length.
     \param s The queue context.
@@ -129,19 +146,22 @@ int queue_read_msg(queue_state_t *s, uint8_t *buf, int len);
     \return The number of bytes actually written. */
 int queue_write_msg(queue_state_t *s, const uint8_t *buf, int len);
 
-/*! Create a queue.
-    \brief Create a queue.
+/*! Initialise a queue.
+    \brief Initialise a queue.
+    \param s The queue context. If is imperative that the context this
+           points to is immediately followed by a buffer of the required
+           size + 1 octet.
     \param len The length of the queue's buffer.
     \param flags Flags controlling the operation of the queue.
            Valid flags are QUEUE_READ_ATOMIC and QUEUE_WRITE_ATOMIC.
-    \return A pointer to the context if created OK, else NULL. */
-queue_state_t *queue_create(int len, int flags);
+    \return A pointer to the context if OK, else NULL. */
+queue_state_t *queue_init(queue_state_t *s, int len, int flags);
 
 /*! Delete a queue.
     \brief Delete a queue.
     \param s The queue context.
     \return 0 if deleted OK, else -1. */
-int queue_delete(queue_state_t *s);
+int queue_free(queue_state_t *s);
 
 #if defined(__cplusplus)
 }
