@@ -1,12 +1,11 @@
-#define ENABLE_V17
 /*
  * SpanDSP - a series of DSP components for telephony
  *
- * t38_gateway.h - An implementation of T.38, less the packet exchange part
+ * t38_gateway.h - A T.38, less the packet exchange part
  *
  * Written by Steve Underwood <steveu@coppice.org>
  *
- * Copyright (C) 2005 Steve Underwood
+ * Copyright (C) 2005, 2006, 2007 Steve Underwood
  *
  * All rights reserved.
  *
@@ -23,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: t38_gateway.h,v 1.27 2007/04/13 12:12:07 steveu Exp $
+ * $Id: t38_gateway.h,v 1.40 2007/12/30 04:45:33 steveu Exp $
  */
 
 /*! \file */
@@ -47,17 +46,23 @@ to maximum the tolerance of jitter and packet loss on the IP network.
 /* Make sure the HDLC frame buffers are big enough for ECM frames. */
 #define T38_MAX_HDLC_LEN        260
 
+/*!
+    T.38 gateway state.
+*/
 typedef struct
 {
+    /*! Core T.38 support */
     t38_core_state_t t38;
 
-    /*! TRUE if ECM FAX mode is allowed through the gateway. */
+    /*! \brief TRUE if ECM FAX mode is allowed through the gateway. */
     int ecm_allowed;
+    /*! \brief Use talker echo protection when transmitting. */
+    int use_tep;    
 
-    /*! If TRUE, transmit silence when there is nothing else to transmit. If FALSE return only
-        the actual generated audio. Note that this only affects untimed silences. Timed silences
-        (e.g. the 75ms silence between V.21 and a high speed modem) will alway be transmitted as
-        silent audio. */
+    /*! \brief If TRUE, transmit silence when there is nothing else to transmit. If FALSE return only
+               the actual generated audio. Note that this only affects untimed silences. Timed silences
+               (e.g. the 75ms silence between V.21 and a high speed modem) will alway be transmitted as
+               silent audio. */
     int transmit_on_idle;
 
     int supported_modems;
@@ -91,7 +96,7 @@ typedef struct
 
     /*! \brief The location of the most recent EOL marker in the non-ECM data buffer */
     int non_ecm_tx_latest_eol_ptr;
-    unsigned int bit_stream;
+    unsigned int non_ecm_bit_stream;
     /*! \brief The non-ECM flow control fill octet (0xFF before the first data, and 0x00
                once data has started). */
     uint8_t non_ecm_flow_control_fill_octet;
@@ -100,9 +105,14 @@ typedef struct
     /*! \brief TRUE is the end of non-ECM data indication has been received. */
     int non_ecm_data_finished;
     /*! \brief The current octet being sent as non-ECM data. */
-    int current_non_ecm_octet;
+    unsigned int non_ecm_rx_bit_stream;
+    unsigned int non_ecm_tx_octet;
     /*! \brief The current bit number in the current non-ECM octet. */
     int non_ecm_bit_no;
+    int non_ecm_in_octets;
+    int non_ecm_out_octets;
+    int non_ecm_in_rows;
+    int non_ecm_out_rows;
     /*! \brief A count of the number of non-ECM fill octets generated for flow control control
                purposes. */
     int non_ecm_flow_control_fill_octets;
@@ -121,10 +131,10 @@ typedef struct
     int fast_bit_rate;
     /*! \brief The current fast modem type. */
     int fast_modem;
-    /*! \brief TRUE if between DCS and TCF */
-    int tcf_in_progress;
-    /*! \brief Use talker echo protection when transmitting. */
-    int use_tep;    
+    /*! \brief TRUE if between DCS and TCF, and we want the fast image modem to
+               start in the T.38 data at a predictable time from the end of the
+               V.21 signal. */
+    int tcf_mode_predictable_modem_start;
     /*! \brief TRUE if a carrier is present. Otherwise FALSE. */
     int rx_signal_present;
     /*! \brief TRUE if a modem has trained correctly. */
@@ -141,26 +151,24 @@ typedef struct
 
     /*! \brief A V.21 FSK modem context used when transmitting HDLC over V.21
                messages. */
-    fsk_tx_state_t v21tx;
+    fsk_tx_state_t v21_tx;
     /*! \brief A V.21 FSK modem context used when receiving HDLC over V.21
                messages. */
-    fsk_rx_state_t v21rx;
+    fsk_rx_state_t v21_rx;
 
-#if defined(ENABLE_V17)
     /*! \brief A V.17 modem context used when sending FAXes at 7200bps, 9600bps
                12000bps or 14400bps*/
-    v17_tx_state_t v17tx;
+    v17_tx_state_t v17_tx;
     /*! \brief A V.29 modem context used when receiving FAXes at 7200bps, 9600bps
                12000bps or 14400bps*/
-    v17_rx_state_t v17rx;
-#endif
+    v17_rx_state_t v17_rx;
 
     /*! \brief A V.29 modem context used when sending FAXes at 7200bps or
                9600bps */
-    v29_tx_state_t v29tx;
+    v29_tx_state_t v29_tx;
     /*! \brief A V.29 modem context used when receiving FAXes at 7200bps or
                9600bps */
-    v29_rx_state_t v29rx;
+    v29_rx_state_t v29_rx;
 
     /*! \brief A V.27ter modem context used when sending FAXes at 2400bps or
                4800bps */
@@ -175,15 +183,21 @@ typedef struct
     /*! \brief Used to insert timed silences. */
     silence_gen_state_t silence_gen;
 
+    /*! \brief The immediately active receive signal handler, which may hop between
+               rx_handler and dummy_rx(). */
+    span_rx_handler_t *immediate_rx_handler;
     /*! \brief The current receive signal handler */
     span_rx_handler_t *rx_handler;
+    /*! \brief An opaque pointer, passed to rx_handler. */
     void *rx_user_data;
 
     /*! \brief The current transmit signal handler */
     span_tx_handler_t *tx_handler;
+    /*! \brief An opaque pointer, passed to tx_handler. */
     void *tx_user_data;
     /*! \brief The transmit signal handler to be used when the current one has finished sending. */
     span_tx_handler_t *next_tx_handler;
+    /*! \brief An opaque pointer, passed to next_tx_handler. */
     void *next_tx_user_data;
 
     /*! \brief The number of octets to send in each image packet (non-ECM or ECM) at the current
@@ -194,7 +208,9 @@ typedef struct
     int fast_rx_active;
     /*! \brief The number of samples until the next timeout event */
     int samples_to_timeout;
-    /*! \brief TRUE is short training is to be used for the fast modem */
+    /*! \brief TRUE if in image data mode (as opposed to TCF mode). */
+    int image_data_mode;
+    /*! \brief TRUE if in image data modem is to use short training. */
     int short_train;
 
     /*! \brief TRUE if we need to corrupt the HDLC frame in progress, so the receiver cannot
@@ -203,10 +219,16 @@ typedef struct
     /*! \brief TRUE if we need to corrupt the HDLC frame in progress, so the receiver cannot
                interpret it. */
     int corrupt_the_frame_from_t38;
-    
+
+    /*! \brief The currently select receiver type */
     int current_rx_type;
+    /*! \brief The currently select transmitter type */
     int current_tx_type;
 
+    /*! \brief Audio logging file handles */
+    int fax_audio_rx_log;
+    int fax_audio_tx_log;
+    /*! \brief Error and flow logging control */
     logging_state_t logging;
 } t38_gateway_state_t;
 
@@ -224,6 +246,12 @@ t38_gateway_state_t *t38_gateway_init(t38_gateway_state_t *s,
                                       t38_tx_packet_handler_t *tx_packet_handler,
                                       void *tx_packet_user_data);
 
+/*! Free a gateway mode T.38 context.
+    \brief Free a T.38 context.
+    \param s The T.38 context.
+    \return 0 for OK, else -1. */
+int t38_gateway_free(t38_gateway_state_t *s);
+
 /*! Process a block of received FAX audio samples.
     \brief Process a block of received FAX audio samples.
     \param s The T.38 context.
@@ -248,12 +276,38 @@ int t38_gateway_tx(t38_gateway_state_t *s, int16_t amp[], int max_len);
 */
 void t38_gateway_set_ecm_capability(t38_gateway_state_t *s, int ecm_allowed);
 
+/*! Select whether silent audio will be sent when transmit is idle.
+    \brief Select whether silent audio will be sent when transmit is idle.
+    \param s The T.38 context.
+    \param transmit_on_idle TRUE if silent audio should be output when the FAX transmitter is
+           idle. FALSE to transmit zero length audio when the FAX transmitter is idle. The default
+           behaviour is FALSE.
+*/
 void t38_gateway_set_transmit_on_idle(t38_gateway_state_t *s, int transmit_on_idle);
 
+/*! Specify which modem types are supported by a T.30 context.
+    \brief Specify supported modems.
+    \param s The T.38 context.
+    \param supported_modems Bit field list of the supported modems.
+*/
 void t38_gateway_set_supported_modems(t38_gateway_state_t *s, int supported_modems);
 
+/*! Select whether NSC, NSF, and NSS should be suppressed. It selected, the contents of
+    these messages are forced to zero for all octets beyond the message type. This makes
+    them look like manufacturer specific messages, from a manufacturer which does not exist.
+    \brief Select whether NSC, NSF, and NSS should be suppressed.
+    \param s The T.38 context.
+    \param suppress_nsx TRUE if NSC, NSF, and NSS should be suppressed.
+*/
 void t38_gateway_set_nsx_suppression(t38_gateway_state_t *s, int suppress_nsx);
 
+/*! Select whether talker echo protection tone will be sent for the image modems.
+    \brief Select whether TEP will be sent for the image modems.
+    \param s The T.38 context.
+    \param use_tep TRUE if TEP should be sent.
+*/
+void t38_gateway_set_tep_mode(t38_gateway_state_t *s, int use_tep);
+
 #if defined(__cplusplus)
 }
 #endif
