@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v22bis_tx.c,v 1.36 2007/05/07 13:11:46 steveu Exp $
+ * $Id: v22bis_tx.c,v 1.38 2008/01/10 14:06:21 steveu Exp $
  */
 
 /*! \file */
@@ -56,6 +56,12 @@
 #include "spandsp/v29rx.h"
 #include "spandsp/v22bis.h"
 
+#if defined(SPANDSP_USE_FIXED_POINT)
+#include "v22bis_tx_fixed_rrc.h"
+#else
+#include "v22bis_tx_floating_rrc.h"
+#endif
+
 /* Quoting from the V.22bis spec.
 
 6.3.1.1 Interworking at 2400 bit/s
@@ -266,456 +272,6 @@ const complexf_t v22bis_constellation[16] =
     { 3.0f, -3.0f}
 };
 
-/* Raised root cosine pulse shaping; Beta = 0.75; 4 symbols either
-   side of the centre. */
-/* Created with mkshape -r 0.0125 0.75 361 -l and then split up */
-#define PULSESHAPER_GAIN            (40.000612087f/40.0f)
-#define PULSESHAPER_COEFF_SETS      40
-
-static const float pulseshaper[PULSESHAPER_COEFF_SETS][V22BIS_TX_FILTER_STEPS] =
-{
-    {
-        -0.0047287346f,         /* Filter 0 */
-        -0.0083947197f,
-        -0.0087380763f,
-         0.0088053673f,
-         0.5108981827f,
-         0.5108981827f,
-         0.0088053673f,
-        -0.0087380763f,
-        -0.0083947197f
-    },
-    {
-        -0.0044638629f,         /* Filter 1 */
-        -0.0089241700f,
-        -0.0111288952f,
-         0.0023412184f,
-         0.5623914901f,
-         0.4599551720f,
-         0.0144817755f,
-        -0.0063186648f,
-        -0.0077293609f
-    },
-    {
-        -0.0041048584f,         /* Filter 2 */
-        -0.0093040596f,
-        -0.0134459768f,
-        -0.0048558766f,
-         0.6141017035f,
-         0.4098822897f,
-         0.0193317049f,
-        -0.0039145680f,
-        -0.0069438567f
-    },
-    {
-        -0.0036565006f,         /* Filter 3 */
-        -0.0095231635f,
-        -0.0156437084f,
-        -0.0127148737f,
-         0.6656848457f,
-         0.3609830295f,
-         0.0233320755f,
-        -0.0015677363f,
-        -0.0060557371f
-    },
-    {
-        -0.0031253709f,         /* Filter 4 */
-        -0.0095729633f,
-        -0.0176768181f,
-        -0.0211485021f,
-         0.7167894869f,
-         0.3135419896f,
-         0.0264748749f,
-         0.0006824956f,
-        -0.0050839319f
-    },
-    {
-        -0.0025197700f,         /* Filter 5 */
-        -0.0094478866f,
-        -0.0195012095f,
-        -0.0300535107f,
-         0.7670600056f,
-         0.2678225635f,
-         0.0287663895f,
-         0.0027999985f,
-        -0.0040483891f
-    },
-    {
-        -0.0018496023f,         /* Filter 6 */
-        -0.0091454978f,
-        -0.0210748106f,
-        -0.0393111426f,
-         0.8161399423f,
-         0.2240649005f,
-         0.0302262769f,
-         0.0047523617f,
-        -0.0029696854f
-    },
-    {
-        -0.0011262266f,         /* Filter 7 */
-        -0.0086666380f,
-        -0.0223584207f,
-        -0.0487878398f,
-         0.8636754069f,
-         0.1824841563f,
-         0.0308864956f,
-         0.0065113237f,
-        -0.0018686358f
-    },
-    {
-        -0.0003622774f,         /* Filter 8 */
-        -0.0080155088f,
-        -0.0233165437f,
-        -0.0583361774f,
-         0.9093185032f,
-         0.1432690480f,
-         0.0307901140f,
-         0.0080531155f,
-        -0.0007659096f
-    },
-    {
-         0.0004285425f,         /* Filter 9 */
-        -0.0071996967f,
-        -0.0239181901f,
-        -0.0677960213f,
-         0.9527307304f,
-         0.1065807242f,
-         0.0299900191f,
-         0.0093587151f,
-         0.0003183408f
-    },
-    {
-         0.0012316933f,         /* Filter 10 */
-        -0.0062301368f,
-        -0.0241376359f,
-        -0.0769959031f,
-         0.9935863233f,
-         0.0725519600f,
-         0.0285475474f,
-         0.0104140102f,
-         0.0013648323f
-    },
-    {
-         0.0020320508f,         /* Filter 11 */
-        -0.0051210137f,
-        -0.0239551212f,
-        -0.0857546018f,
-         1.0315754934f,
-         0.0412866769f,
-         0.0265310587f,
-         0.0112098711f,
-         0.0023554782f
-    },
-    {
-         0.0028141763f,         /* Filter 12 */
-        -0.0038896008f,
-        -0.0233574765f,
-        -0.0938829156f,
-         1.0664075323f,
-         0.0128597894f,
-         0.0240144782f,
-         0.0117421344f,
-         0.0032736852f
-    },
-    {
-         0.0035625973f,         /* Filter 13 */
-        -0.0025560369f,
-        -0.0223386620f,
-        -0.1011856112f,
-         1.0978137424f,
-        -0.0126826277f,
-         0.0210758250f,
-         0.0120115019f,
-         0.0041046165f
-    },
-    {
-         0.0042620971f,         /* Filter 14 */
-        -0.0011430452f,
-        -0.0209002083f,
-        -0.1074635265f,
-         1.1255501596f,
-        -0.0353228594f,
-         0.0177957527f,
-         0.0120233576f,
-         0.0048354157f
-    },
-    {
-         0.0048980053f,         /* Filter 15 */
-         0.0003244045f,
-        -0.0190515462f,
-        -0.1125158080f,
-         1.1494000377f,
-        -0.0550707703f,
-         0.0142561191f,
-         0.0117875105f,
-         0.0054553898f
-    },
-    {
-         0.0054564857f,         /* Filter 16 */
-         0.0018194852f,
-        -0.0168102168f,
-        -0.1161422557f,
-         1.1691760633f,
-        -0.0719627404f,
-         0.0105386068f,
-         0.0113178688f,
-         0.0059561483f
-    },
-    {
-         0.0059248154f,         /* Filter 17 */
-         0.0033139475f,
-        -0.0142019526f,
-        -0.1181457502f,
-         1.1847222770f,
-        -0.0860603350f,
-         0.0067234125f,
-         0.0106320540f,
-         0.0063316975f
-    },
-    {
-         0.0062916504f,         /* Filter 18 */
-         0.0047785946f,
-        -0.0112606234f,
-        -0.1183347329f,
-         1.1959156771f,
-        -0.0974487311f,
-         0.0028880206f,
-         0.0097509621f,
-         0.0065784888f
-    },
-    {
-         0.0065472715f,         /* Filter 19 */
-         0.0061837898f,
-        -0.0080280420f,
-        -0.1165257094f,
-         1.2026674866f,
-        -0.1062349247f,
-        -0.0008939235f,
-         0.0086982833f,
-         0.0066954225f
-    },
-    {
-         0.0066838062f,         /* Filter 20 */
-         0.0074999881f,
-        -0.0045536271f,
-        -0.1125457458f,
-         1.2049240699f,
-        -0.1125457458f,
-        -0.0045536271f,
-         0.0074999881f,
-         0.0066838062f
-    },
-    {
-         0.0066954225f,         /* Filter 21 */
-         0.0086982833f,
-        -0.0008939235f,
-        -0.1062349247f,
-         1.2026674866f,
-        -0.1165257094f,
-        -0.0080280420f,
-         0.0061837898f,
-         0.0065472715f
-    },
-    {
-         0.0065784888f,         /* Filter 22 */
-         0.0097509621f,
-         0.0028880206f,
-        -0.0974487311f,
-         1.1959156771f,
-        -0.1183347329f,
-        -0.0112606234f,
-         0.0047785946f,
-         0.0062916504f
-    },
-    {
-         0.0063316975f,         /* Filter 23 */
-         0.0106320540f,
-         0.0067234125f,
-        -0.0860603350f,
-         1.1847222770f,
-        -0.1181457502f,
-        -0.0142019526f,
-         0.0033139475f,
-         0.0059248154f
-    },
-    {
-         0.0059561483f,         /* Filter 24 */
-         0.0113178688f,
-         0.0105386068f,
-        -0.0719627404f,
-         1.1691760633f,
-        -0.1161422557f,
-        -0.0168102168f,
-         0.0018194852f,
-         0.0054564857f
-    },
-    {
-         0.0054553898f,         /* Filter 25 */
-         0.0117875105f,
-         0.0142561191f,
-        -0.0550707703f,
-         1.1494000377f,
-        -0.1125158080f,
-        -0.0190515462f,
-         0.0003244045f,
-         0.0048980053f
-    },
-    {
-         0.0048354157f,         /* Filter 26 */
-         0.0120233576f,
-         0.0177957527f,
-        -0.0353228594f,
-         1.1255501596f,
-        -0.1074635265f,
-        -0.0209002083f,
-        -0.0011430452f,
-         0.0042620971f
-    },
-    {
-         0.0041046165f,         /* Filter 27 */
-         0.0120115019f,
-         0.0210758250f,
-        -0.0126826277f,
-         1.0978137424f,
-        -0.1011856112f,
-        -0.0223386620f,
-        -0.0025560369f,
-         0.0035625973f
-    },
-    {
-         0.0032736852f,         /* Filter 28 */
-         0.0117421344f,
-         0.0240144782f,
-         0.0128597894f,
-         1.0664075323f,
-        -0.0938829156f,
-        -0.0233574765f,
-        -0.0038896008f,
-         0.0028141763f
-    },
-    {
-         0.0023554782f,         /* Filter 29 */
-         0.0112098711f,
-         0.0265310587f,
-         0.0412866769f,
-         1.0315754934f,
-        -0.0857546018f,
-        -0.0239551212f,
-        -0.0051210137f,
-         0.0020320508f
-    },
-    {
-         0.0013648323f,         /* Filter 30 */
-         0.0104140102f,
-         0.0285475474f,
-         0.0725519600f,
-         0.9935863233f,
-        -0.0769959031f,
-        -0.0241376359f,
-        -0.0062301368f,
-         0.0012316933f
-    },
-    {
-         0.0003183408f,         /* Filter 31 */
-         0.0093587151f,
-         0.0299900191f,
-         0.1065807242f,
-         0.9527307304f,
-        -0.0677960213f,
-        -0.0239181901f,
-        -0.0071996967f,
-         0.0004285425f
-    },
-    {
-        -0.0007659096f,         /* Filter 32 */
-         0.0080531155f,
-         0.0307901140f,
-         0.1432690480f,
-         0.9093185032f,
-        -0.0583361774f,
-        -0.0233165437f,
-        -0.0080155088f,
-        -0.0003622774f
-    },
-    {
-        -0.0018686358f,         /* Filter 33 */
-         0.0065113237f,
-         0.0308864956f,
-         0.1824841563f,
-         0.8636754069f,
-        -0.0487878398f,
-        -0.0223584207f,
-        -0.0086666380f,
-        -0.0011262266f
-    },
-    {
-        -0.0029696854f,         /* Filter 34 */
-         0.0047523617f,
-         0.0302262769f,
-         0.2240649005f,
-         0.8161399423f,
-        -0.0393111426f,
-        -0.0210748106f,
-        -0.0091454978f,
-        -0.0018496023f
-    },
-    {
-        -0.0040483891f,         /* Filter 35 */
-         0.0027999985f,
-         0.0287663895f,
-         0.2678225635f,
-         0.7670600056f,
-        -0.0300535107f,
-        -0.0195012095f,
-        -0.0094478866f,
-        -0.0025197700f
-    },
-    {
-        -0.0050839319f,         /* Filter 36 */
-         0.0006824956f,
-         0.0264748749f,
-         0.3135419896f,
-         0.7167894869f,
-        -0.0211485021f,
-        -0.0176768181f,
-        -0.0095729633f,
-        -0.0031253709f
-    },
-    {
-        -0.0060557371f,         /* Filter 37 */
-        -0.0015677363f,
-         0.0233320755f,
-         0.3609830295f,
-         0.6656848457f,
-        -0.0127148737f,
-        -0.0156437084f,
-        -0.0095231635f,
-        -0.0036565006f
-    },
-    {
-        -0.0069438567f,         /* Filter 38 */
-        -0.0039145680f,
-         0.0193317049f,
-         0.4098822897f,
-         0.6141017035f,
-        -0.0048558766f,
-        -0.0134459768f,
-        -0.0093040596f,
-        -0.0041048584f
-    },
-    {
-        -0.0077293609f,         /* Filter 39 */
-        -0.0063186648f,
-         0.0144817755f,
-         0.4599551720f,
-         0.5623914901f,
-         0.0023412184f,
-        -0.0111288952f,
-        -0.0089241700f,
-        -0.0044638629f
-    },
-};
-
 static int fake_get_bit(void *user_data)
 {
     return 1;
@@ -1068,6 +624,8 @@ v22bis_state_t *v22bis_init(v22bis_state_t *s,
             return NULL;
     }
     memset(s, 0, sizeof(*s));
+    span_log_init(&s->logging, SPAN_LOG_NONE, NULL);
+    span_log_set_protocol(&s->logging, "V.22bis");
     s->bit_rate = bit_rate;
     s->caller = caller;
 
@@ -1097,8 +655,6 @@ v22bis_state_t *v22bis_init(v22bis_state_t *s,
         }
     }
     v22bis_tx_power(s, -10.0f);
-    span_log_init(&s->logging, SPAN_LOG_NONE, NULL);
-    span_log_set_protocol(&s->logging, "V.22bis");
     v22bis_restart(s, s->bit_rate);
     return s;
 }
