@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v27ter_tx.c,v 1.46 2006/10/24 13:45:27 steveu Exp $
+ * $Id: v27ter_tx.c,v 1.57 2008/01/10 14:06:21 steveu Exp $
  */
 
 /*! \file */
@@ -35,7 +35,12 @@
 #include <inttypes.h>
 #include <stdlib.h>
 #include <string.h>
+#if defined(HAVE_TGMATH_H)
 #include <tgmath.h>
+#endif
+#if defined(HAVE_MATH_H)
+#include <math.h>
+#endif
 
 #include "spandsp/telephony.h"
 #include "spandsp/logging.h"
@@ -48,6 +53,14 @@
 
 #include "spandsp/v27ter_tx.h"
 
+#if defined(SPANDSP_USE_FIXED_POINT)
+#include "v27ter_tx_4800_fixed_rrc.h"
+#include "v27ter_tx_2400_fixed_rrc.h"
+#else
+#include "v27ter_tx_4800_floating_rrc.h"
+#include "v27ter_tx_2400_floating_rrc.h"
+#endif
+
 #define CARRIER_NOMINAL_FREQ            1800.0f
 
 /* Segments of the training sequence */
@@ -61,298 +74,9 @@
 #define V27TER_TRAINING_END             (V27TER_TRAINING_SEG_5 + 8)
 #define V27TER_TRAINING_SHUTDOWN_END    (V27TER_TRAINING_END + 32)
 
-/* Raised root cosine pulse shaping; Beta = 0.5; 4 symbols either
-   side of the centre. */
-/* Created with mkshape -r 0.025 0.5 181 -l and then split up */
 #define PULSESHAPER_2400_GAIN           (19.972065748f/20.0f)
-#define PULSESHAPER_2400_COEFF_SETS     20
-static const float pulseshaper_2400[PULSESHAPER_2400_COEFF_SETS][V27TER_TX_FILTER_STEPS] =
-{
-    {
-         0.0050051219,          /* Filter 0 */
-         0.0107180844,
-        -0.0150077814,
-        -0.0750272071,
-         0.5786341413,
-         0.5786341413,
-        -0.0750272071,
-        -0.0150077814,
-         0.0107180844
-    },
-    {
-         0.0036624469,          /* Filter 1 */
-         0.0131516633,
-        -0.0107913392,
-        -0.0957820135,
-         0.6671466059,
-         0.4891745311,
-        -0.0541239470,
-        -0.0179109014,
-         0.0079099936
-    },
-    {
-         0.0020204744,          /* Filter 2 */
-         0.0150588729,
-        -0.0053908083,
-        -0.1154114754,
-         0.7528295479,
-         0.4006032722,
-        -0.0339459430,
-        -0.0194500407,
-         0.0048904515
-    },
-    {
-         0.0001596234,          /* Filter 3 */
-         0.0163079778,
-         0.0009858079,
-        -0.1328632049,
-         0.8338068363,
-         0.3146585634,
-        -0.0152415667,
-        -0.0196492903,
-         0.0018247182
-    },
-    {
-        -0.0018233575,          /* Filter 4 */
-         0.0167957238,
-         0.0080554403,
-        -0.1470417557,
-         0.9082626683,
-         0.2329352195,
-         0.0013822552,
-        -0.0186004475,
-        -0.0011283792
-    },
-    {
-        -0.0038199491,          /* Filter 5 */
-         0.0164546659,
-         0.0154676597,
-        -0.1568448230,
-         0.9744947974,
-         0.1568443643,
-         0.0154698286,
-        -0.0164532877,
-        -0.0038242967
-    },
-    {
-        -0.0057152767,          /* Filter 6 */
-         0.0152590213,
-         0.0228163087,
-        -0.1612020164,
-         1.0309651039,
-         0.0875801110,
-         0.0267201501,
-        -0.0134037738,
-        -0.0061394831
-    },
-    {
-        -0.0073941287,          /* Filter 7 */
-         0.0132286539,
-         0.0296547979,
-        -0.1591148676,
-         1.0763457753,
-         0.0260941722,
-         0.0349842710,
-        -0.0096808822,
-        -0.0079766730
-    },
-    {
-        -0.0087472825,          /* Filter 8 */
-         0.0104308721,
-         0.0355146231,
-        -0.1496966290,
-         1.1095595051,
-        -0.0269209682,
-         0.0402570324,
-        -0.0055327477,
-        -0.0092685626
-    },
-    {
-        -0.0096778115,          /* Filter 9 */
-         0.0069798134,
-         0.0399264862,
-        -0.1322103702,
-         1.1298123136,
-        -0.0710400038,
-         0.0426638320,
-        -0.0012128224,
-        -0.0099797659
-    },
-    {
-        -0.0101070340,          /* Filter 10 */
-         0.0030333009,
-         0.0424432507,
-        -0.1061038872,
-         1.1366178484,
-        -0.1061038872,
-         0.0424432507,
-         0.0030333009,
-        -0.0101070340
-    },
-    {
-        -0.0099797659,          /* Filter 11 */
-        -0.0012128224,
-         0.0426638320,
-        -0.0710400038,
-         1.1298123136,
-        -0.1322103702,
-         0.0399264862,
-         0.0069798134,
-        -0.0096778115
-    },
-    {
-        -0.0092685626,          /* Filter 12 */
-        -0.0055327477,
-         0.0402570324,
-        -0.0269209682,
-         1.1095595051,
-        -0.1496966290,
-         0.0355146231,
-         0.0104308721,
-        -0.0087472825
-    },
-    {
-        -0.0079766730,          /* Filter 13 */
-        -0.0096808822,
-         0.0349842710,
-         0.0260941722,
-         1.0763457753,
-        -0.1591148676,
-         0.0296547979,
-         0.0132286539,
-        -0.0073941287
-    },
-    {
-        -0.0061394831,          /* Filter 14 */
-        -0.0134037738,
-         0.0267201501,
-         0.0875801110,
-         1.0309651039,
-        -0.1612020164,
-         0.0228163087,
-         0.0152590213,
-        -0.0057152767
-    },
-    {
-        -0.0038242967,          /* Filter 15 */
-        -0.0164532877,
-         0.0154698286,
-         0.1568443643,
-         0.9744947974,
-        -0.1568448230,
-         0.0154676597,
-         0.0164546659,
-        -0.0038199491
-    },
-    {
-        -0.0011283792,          /* Filter 16 */
-        -0.0186004475,
-         0.0013822552,
-         0.2329352195,
-         0.9082626683,
-        -0.1470417557,
-         0.0080554403,
-         0.0167957238,
-        -0.0018233575
-    },
-    {
-         0.0018247182,          /* Filter 17 */
-        -0.0196492903,
-        -0.0152415667,
-         0.3146585634,
-         0.8338068363,
-        -0.1328632049,
-         0.0009858079,
-         0.0163079778,
-         0.0001596234
-    },
-    {
-         0.0048904515,          /* Filter 18 */
-        -0.0194500407,
-        -0.0339459430,
-         0.4006032722,
-         0.7528295479,
-        -0.1154114754,
-        -0.0053908083,
-         0.0150588729,
-         0.0020204744
-    },
-    {
-         0.0079099936,          /* Filter 19 */
-        -0.0179109014,
-        -0.0541239470,
-         0.4891745311,
-         0.6671466059,
-        -0.0957820135,
-        -0.0107913392,
-         0.0131516633,
-         0.0036624469
-    },
-};
 
-/* Raised root cosine pulse shaping; Beta = 0.5; 4 symbols either
-   side of the centre. */
-/* Created with mkshape -r 0.1 0.5 45 -l and then split up */
 #define PULSESHAPER_4800_GAIN           (4.9913162900f/5.0f)
-#define PULSESHAPER_4800_COEFF_SETS     5
-static const float pulseshaper_4800[PULSESHAPER_4800_COEFF_SETS][V27TER_TX_FILTER_STEPS] =
-{
-    {
-         0.0020173211,          /* Filter 0 */
-         0.0150576434,
-        -0.0053888047,
-        -0.1154099010,
-         0.7528286821,
-         0.4006013374,
-        -0.0339462085,
-        -0.0194477281,
-         0.0048918464
-    },
-    {
-        -0.0057162575,          /* Filter 1 */
-         0.0152563286,
-         0.0228163350,
-        -0.1612000503,
-         1.0309660372,
-         0.0875788553,
-         0.0267182476,
-        -0.0134032156,
-        -0.0061365979
-    },
-    {
-        -0.0101052019,          /* Filter 2 */
-         0.0030314952,
-         0.0424414442,
-        -0.1061032862,
-         1.1366196464,
-        -0.1061032862,
-         0.0424414442,
-         0.0030314952,
-        -0.0101052019
-    },
-    {
-        -0.0061365979,          /* Filter 3 */
-        -0.0134032156,
-         0.0267182476,
-         0.0875788553,
-         1.0309660372,
-        -0.1612000503,
-         0.0228163350,
-         0.0152563286,
-        -0.0057162575
-    },
-    {
-         0.0048918464,          /* Filter 4 */
-        -0.0194477281,
-        -0.0339462085,
-         0.4006013374,
-         0.7528286821,
-        -0.1154099010,
-        -0.0053888047,
-         0.0150576434,
-         0.0020173211
-    },
-};
 
 static int fake_get_bit(void *user_data)
 {
@@ -399,7 +123,11 @@ static __inline__ int get_scrambled_bit(v27ter_tx_state_t *s)
 }
 /*- End of function --------------------------------------------------------*/
 
+#if defined(SPANDSP_USE_FIXED_POINT)
+static complexi16_t getbaud(v27ter_tx_state_t *s)
+#else
 static complexf_t getbaud(v27ter_tx_state_t *s)
+#endif
 {
     static const int phase_steps_4800[8] =
     {
@@ -409,6 +137,19 @@ static complexf_t getbaud(v27ter_tx_state_t *s)
     {
         0, 2, 6, 4
     };
+#if defined(SPANDSP_USE_FIXED_POINT)
+    static const complexi16_t constellation[8] =
+    {
+        { 1414,    0000},       /*   0deg */
+        { 1000,    1000},       /*  45deg */
+        { 0000,    1414},       /*  90deg */
+        {-1000,    1000},       /* 135deg */
+        {-1414,    0000},       /* 180deg */
+        {-1000,   -1000},       /* 225deg */
+        { 0000,   -1414},       /* 270deg */
+        { 1000,   -1000}        /* 315deg */
+    };
+#else
     static const complexf_t constellation[8] =
     {
         { 1.414f,  0.0f},       /*   0deg */
@@ -420,6 +161,7 @@ static complexf_t getbaud(v27ter_tx_state_t *s)
         { 0.0f,   -1.414f},     /* 270deg */
         { 1.0f,   -1.0f}        /* 315deg */
     };
+#endif
     int bits;
 
     if (s->in_training)
@@ -437,7 +179,11 @@ static complexf_t getbaud(v27ter_tx_state_t *s)
                 if (s->training_step <= V27TER_TRAINING_SEG_3)
                 {
                     /* Segment 2: Silence */
-                    return complex_setf(0.0, 0.0);
+#if defined(SPANDSP_USE_FIXED_POINT)
+                    return complex_seti16(0, 0);
+#else
+                    return complex_setf(0.0f, 0.0f);
+#endif
                 }
                 /* Segment 3: Regular reversals... */
                 s->constellation_state = (s->constellation_state + 4) & 7;
@@ -486,8 +232,13 @@ static complexf_t getbaud(v27ter_tx_state_t *s)
 
 int v27ter_tx(v27ter_tx_state_t *s, int16_t amp[], int len)
 {
+#if defined(SPANDSP_USE_FIXED_POINT)
+    complexi_t x;
+    complexi_t z;
+#else
     complexf_t x;
     complexf_t z;
+#endif
     int i;
     int sample;
 
@@ -512,8 +263,22 @@ int v27ter_tx(v27ter_tx_state_t *s, int16_t amp[], int len)
                     s->rrc_filter_step = 0;
             }
             /* Root raised cosine pulse shaping at baseband */
-            x.re = 0.0f;
-            x.im = 0.0f;
+#if defined(SPANDSP_USE_FIXED_POINT)
+            x = complex_seti(0, 0);
+            for (i = 0;  i < V27TER_TX_FILTER_STEPS;  i++)
+            {
+                x.re += (int32_t) pulseshaper_4800[4 - s->baud_phase][i]*(int32_t) s->rrc_filter[i + s->rrc_filter_step].re;
+                x.im += (int32_t) pulseshaper_4800[4 - s->baud_phase][i]*(int32_t) s->rrc_filter[i + s->rrc_filter_step].im;
+            }
+            /* Now create and modulate the carrier */
+            x.re >>= 14;
+            x.im >>= 14;
+            z = dds_complexi(&(s->carrier_phase), s->carrier_phase_rate);
+            /* Don't bother saturating. We should never clip. */
+            i = (x.re*z.re - x.im*z.im) >> 15;
+            amp[sample] = (int16_t) ((i*s->gain_4800) >> 15);
+#else
+            x = complex_setf(0.0f, 0.0f);
             for (i = 0;  i < V27TER_TX_FILTER_STEPS;  i++)
             {
                 x.re += pulseshaper_4800[4 - s->baud_phase][i]*s->rrc_filter[i + s->rrc_filter_step].re;
@@ -522,7 +287,8 @@ int v27ter_tx(v27ter_tx_state_t *s, int16_t amp[], int len)
             /* Now create and modulate the carrier */
             z = dds_complexf(&(s->carrier_phase), s->carrier_phase_rate);
             /* Don't bother saturating. We should never clip. */
-            amp[sample] = (int16_t) lrintf((x.re*z.re - x.im*z.im)*s->gain_4800);
+            amp[sample] = (int16_t) rintf((x.re*z.re - x.im*z.im)*s->gain_4800);
+#endif
         }
     }
     else
@@ -538,8 +304,22 @@ int v27ter_tx(v27ter_tx_state_t *s, int16_t amp[], int len)
                     s->rrc_filter_step = 0;
             }
             /* Root raised cosine pulse shaping at baseband */
-            x.re = 0.0f;
-            x.im = 0.0f;
+#if defined(SPANDSP_USE_FIXED_POINT)
+            x = complex_seti(0, 0);
+            for (i = 0;  i < V27TER_TX_FILTER_STEPS;  i++)
+            {
+                x.re += (int32_t) pulseshaper_2400[19 - s->baud_phase][i]*(int32_t) s->rrc_filter[i + s->rrc_filter_step].re;
+                x.im += (int32_t) pulseshaper_2400[19 - s->baud_phase][i]*(int32_t) s->rrc_filter[i + s->rrc_filter_step].im;
+            }
+            /* Now create and modulate the carrier */
+            x.re >>= 14;
+            x.im >>= 14;
+            z = dds_complexi(&(s->carrier_phase), s->carrier_phase_rate);
+            /* Don't bother saturating. We should never clip. */
+            i = (x.re*z.re - x.im*z.im) >> 15;
+            amp[sample] = (int16_t) ((i*s->gain_2400) >> 15);
+#else
+            x = complex_setf(0.0f, 0.0f);
             for (i = 0;  i < V27TER_TX_FILTER_STEPS;  i++)
             {
                 x.re += pulseshaper_2400[19 - s->baud_phase][i]*s->rrc_filter[i + s->rrc_filter_step].re;
@@ -548,7 +328,8 @@ int v27ter_tx(v27ter_tx_state_t *s, int16_t amp[], int len)
             /* Now create and modulate the carrier */
             z = dds_complexf(&(s->carrier_phase), s->carrier_phase_rate);
             /* Don't bother saturating. We should never clip. */
-            amp[sample] = (int16_t) lrintf((x.re*z.re - x.im*z.im)*s->gain_2400);
+            amp[sample] = (int16_t) rintf((x.re*z.re - x.im*z.im)*s->gain_2400);
+#endif
         }
     }
     return sample;
@@ -560,8 +341,13 @@ void v27ter_tx_power(v27ter_tx_state_t *s, float power)
     float l;
 
     l = powf(10.0f, (power - DBM0_MAX_POWER)/20.0f)*32768.0f;
+#if defined(SPANDSP_USE_FIXED_POINT)
+    s->gain_2400 = 16.0f*1.024f*(32767.0f/28828.51f)*l/PULSESHAPER_2400_GAIN;
+    s->gain_4800 = 16.0f*1.024f*(32767.0f/28828.46f)*l/PULSESHAPER_4800_GAIN;
+#else
     s->gain_2400 = l/PULSESHAPER_2400_GAIN;
     s->gain_4800 = l/PULSESHAPER_4800_GAIN;
+#endif
 }
 /*- End of function --------------------------------------------------------*/
 
@@ -579,7 +365,11 @@ int v27ter_tx_restart(v27ter_tx_state_t *s, int rate, int tep)
     if (rate != 4800  &&  rate != 2400)
         return -1;
     s->bit_rate = rate;
+#if defined(SPANDSP_USE_FIXED_POINT)
+    memset(s->rrc_filter, 0, sizeof(s->rrc_filter));
+#else
     cvec_zerof(s->rrc_filter, sizeof(s->rrc_filter)/sizeof(s->rrc_filter[0]));
+#endif
     s->rrc_filter_step = 0;
     s->scramble_reg = 0x3C;
     s->scrambler_pattern_count = 0;
@@ -601,6 +391,8 @@ v27ter_tx_state_t *v27ter_tx_init(v27ter_tx_state_t *s, int rate, int tep, get_b
             return NULL;
     }
     memset(s, 0, sizeof(*s));
+    span_log_init(&s->logging, SPAN_LOG_NONE, NULL);
+    span_log_set_protocol(&s->logging, "V.27ter TX");
     s->get_bit = get_bit;
     s->user_data = user_data;
     s->carrier_phase_rate = dds_phase_ratef(CARRIER_NOMINAL_FREQ);
@@ -610,7 +402,7 @@ v27ter_tx_state_t *v27ter_tx_init(v27ter_tx_state_t *s, int rate, int tep, get_b
 }
 /*- End of function --------------------------------------------------------*/
 
-int v27ter_tx_release(v27ter_tx_state_t *s)
+int v27ter_tx_free(v27ter_tx_state_t *s)
 {
     free(s);
     return 0;
