@@ -24,15 +24,16 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: g168_tests.c,v 1.8 2006/10/24 13:45:29 steveu Exp $
+ * $Id: g168_tests.c,v 1.11 2007/11/10 11:14:58 steveu Exp $
  */
 
-#include <inttypes.h>
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <string.h>
-#include <tgmath.h>
-#include <tiffio.h>
 
 #include "spandsp.h"
 #include "spandsp/g168models.h"
