@@ -23,7 +23,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: ima_adpcm_tests.c,v 1.21 2006/10/24 13:45:29 steveu Exp $
+ * $Id: ima_adpcm_tests.c,v 1.23 2007/11/10 11:14:58 steveu Exp $
  */
 
 /*! \file */
@@ -44,14 +44,11 @@ of the degradation in quality caused by the compression.
 #endif
 
 #include <stdlib.h>
-#include <math.h>
-#include <inttypes.h>
-#include <string.h>
 #include <stdio.h>
-#include <time.h>
 #include <fcntl.h>
+#include <string.h>
+#include <time.h>
 #include <audiofile.h>
-#include <tiffio.h>
 
 #include "spandsp.h"
 
