@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: queue_tests.c,v 1.2 2007/05/02 14:26:29 steveu Exp $
+ * $Id: queue_tests.c,v 1.6 2007/11/30 12:20:36 steveu Exp $
  */
 
 /* THIS IS A WORK IN PROGRESS. IT IS NOT FINISHED. */
@@ -36,18 +36,10 @@
 #endif
 
 #include <stdlib.h>
-#include <unistd.h>
 #include <stdio.h>
-#include <inttypes.h>
+#include <unistd.h>
 #include <string.h>
-#if defined(HAVE_TGMATH_H)
-#include <tgmath.h>
-#endif
-#if defined(HAVE_MATH_H)
-#include <math.h>
-#endif
 #include <assert.h>
-#include <tiffio.h>
 #include <pthread.h>
 #include <sched.h>
 
@@ -66,6 +58,19 @@ volatile int got_misses;
 int total_in;
 int total_out;
 
+static void tests_failed(void)
+{
+    printf("Tests failed\n");
+    tests_failed();
+}
+/*- End of function --------------------------------------------------------*/
+
+static void display_queue_pointers(void)
+{
+    printf("Pointers %d %d %d\n", queue->iptr, queue->optr, queue->len);
+}
+/*- End of function --------------------------------------------------------*/
+
 static void *run_stream_write(void *arg)
 {
     uint8_t buf[MSG_LEN];
@@ -118,14 +123,14 @@ static void *run_stream_read(void *arg)
             if (len != MSG_LEN)
             {
                 printf("AHH! - len %d\n", len);
-                exit(2);
+                tests_failed();
             }
             for (i = 0;  i < len;  i++)
             {
                 if (buf[i] != next)
                 {
                     printf("AHH! - 0x%X 0x%X\n", buf[i], next);
-                    exit(2);
+                    tests_failed();
                 }
             }
             next = (next + 1) & 0xFF;
@@ -147,29 +152,29 @@ static void threaded_stream_tests(void)
 {
     pthread_attr_t attr;
 
-    if ((queue = queue_create(BUF_LEN, QUEUE_READ_ATOMIC | QUEUE_WRITE_ATOMIC)) == NULL)
+    if ((queue = queue_init(NULL, BUF_LEN, QUEUE_READ_ATOMIC | QUEUE_WRITE_ATOMIC)) == NULL)
     {
         printf("Failed to create the queue\n");
-        exit(2);
+        tests_failed();
     }
     pthread_attr_init(&attr);
     pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
     if (pthread_create(&thread[0], &attr, run_stream_write, NULL))
     {
         printf("Failed to create thread\n");
-        exit(2);
+        tests_failed();
     }
     if (pthread_create(&thread[1], &attr, run_stream_read, NULL))
     {
         printf("Failed to create thread\n");
-        exit(2);
+        tests_failed();
     }
     for (;;)
     {
         sleep(5);
         printf("Main thread - %d %d\n", put_oks, got_oks);
     }
-    queue_delete(queue);
+    queue_free(queue);
 }
 /*- End of function --------------------------------------------------------*/
 
@@ -225,14 +230,14 @@ static void *run_message_read(void *arg)
             if (len != MSG_LEN)
             {
                 printf("AHH! - len %d\n", len);
-                exit(2);
+                tests_failed();
             }
             for (i = 0;  i < len;  i++)
             {
                 if (buf[i] != next)
                 {
                     printf("AHH! - 0x%X 0x%X\n", buf[i], next);
-                    exit(2);
+                    tests_failed();
                 }
             }
             next = (next + 1) & 0xFF;
@@ -254,29 +259,29 @@ static void threaded_message_tests(void)
 {
     pthread_attr_t attr;
 
-    if ((queue = queue_create(BUF_LEN, QUEUE_READ_ATOMIC | QUEUE_WRITE_ATOMIC)) == NULL)
+    if ((queue = queue_init(NULL, BUF_LEN, QUEUE_READ_ATOMIC | QUEUE_WRITE_ATOMIC)) == NULL)
     {
         printf("Failed to create the queue\n");
-        exit(2);
+        tests_failed();
     }
     pthread_attr_init(&attr);
     pthread_attr_setdetachstate(&attr, PTHREAD_CREATE_DETACHED);
     if (pthread_create(&thread[0], &attr, run_message_write, NULL))
     {
         printf("Failed to create thread\n");
-        exit(2);
+        tests_failed();
     }
     if (pthread_create(&thread[1], &attr, run_message_read, NULL))
     {
         printf("Failed to create thread\n");
-        exit(2);
+        tests_failed();
     }
     for (;;)
     {
         sleep(5);
         printf("Main thread - %d %d\n", put_oks, got_oks);
     }
-    queue_delete(queue);
+    queue_free(queue);
 }
 /*- End of function --------------------------------------------------------*/
 
@@ -285,14 +290,14 @@ static void check_contents(int total_in, int total_out)
     if (queue_contents(queue) != (total_in - total_out))
     {
         printf("Contents = %d (%d)\n", queue_contents(queue), (total_in - total_out));
-        printf("Pointers: %d %d %d\n", queue->iptr, queue->optr, queue->len);
-        exit(2);
+        display_queue_pointers();
+        tests_failed();
     }
     if (queue_free_space(queue) != BUF_LEN - (total_in - total_out))
     {
         printf("Free space = %d (%d)\n", queue_free_space(queue), BUF_LEN - (total_in - total_out));
-        printf("Pointers: %d %d %d\n", queue->iptr, queue->optr, queue->len);
-        exit(2);
+        display_queue_pointers();
+        tests_failed();
     }
 }
 /*- End of function --------------------------------------------------------*/
@@ -309,6 +314,17 @@ static int monitored_queue_write(const uint8_t buf[], int len)
 }
 /*- End of function --------------------------------------------------------*/
 
+static int monitored_queue_write_byte(const uint8_t buf)
+{
+    int res;
+    
+    if ((res = queue_write_byte(queue, buf)) >= 0)
+        total_in++;
+    check_contents(total_in, total_out);
+    return res;
+}
+/*- End of function --------------------------------------------------------*/
+
 static int monitored_queue_read(uint8_t buf[], int len)
 {
     int lenx;
@@ -321,22 +337,69 @@ static int monitored_queue_read(uint8_t buf[], int len)
 }
 /*- End of function --------------------------------------------------------*/
 
+static int monitored_queue_read_byte(void)
+{
+    int res;
+    
+    if ((res = queue_read_byte(queue)) >= 0)
+        total_out++;
+    check_contents(total_in, total_out);
+    return res;
+}
+/*- End of function --------------------------------------------------------*/
+
 static void functional_stream_tests(void)
 {
     uint8_t buf[MSG_LEN];
     int i;
+    int res;
     
     total_in = 0;
     total_out = 0;
 
     for (i = 0;  i < MSG_LEN;  i++)
         buf[i] = i;
-    if ((queue = queue_create(BUF_LEN, QUEUE_READ_ATOMIC | QUEUE_WRITE_ATOMIC)) == NULL)
+    if ((queue = queue_init(NULL, BUF_LEN, QUEUE_READ_ATOMIC | QUEUE_WRITE_ATOMIC)) == NULL)
     {
         printf("Failed to create the queue\n");
-        exit(2);
+        tests_failed();
     }
     check_contents(total_in, total_out);
+    /* Half fill the buffer, and check we can get out what we put in. */
+    for (i = 1;  i < 5000;  i++)
+    {
+        if (monitored_queue_write_byte(i & 0xFF) != 1)
+        {
+            printf("Byte by byte full at %d/%d\n", i, BUF_LEN);
+            tests_failed();
+        }
+    }
+    for (i = 1;  i < 5001;  i++)
+    {
+        if ((res = monitored_queue_read_byte()) != (i & 0xFF))
+            break;
+    }
+    printf("Byte by byte read breaks at %d (expected %d) - %d\n", i, 5000, res);
+    if (i != 5000)
+        tests_failed();
+    /* Now completely fill the buffer, and we should roll around the end. Check we can
+       get out what we put in. */
+    for (i = 1;  i < 20000;  i++)
+    {
+        if (monitored_queue_write_byte(i & 0xFF) != 1)
+            break;
+    }
+    printf("Byte by byte full at %d (expected %d)\n", i, 10001);
+    if (i != 10001)
+        tests_failed();
+    for (i = 1;  i < 20000;  i++)
+    {
+        if ((res = monitored_queue_read_byte()) != (i & 0xFF))
+            break;
+    }
+    printf("Byte by byte read breaks at %d (expected %d) - %d\n", i, 10001, res);
+    if (i != 10001)
+        tests_failed();
     /* Fill the buffer, checking the contents grow correctly */
     for (i = 1;  i < 1000;  i++)
     {
@@ -344,26 +407,28 @@ static void functional_stream_tests(void)
             break;
     }
     printf("Full at chunk %d (expected %d)\n", i, BUF_LEN/MSG_LEN + 1);
+    if (i != BUF_LEN/MSG_LEN + 1)
+        tests_failed();
     if (monitored_queue_write(buf, 5) == 5)
     {
         printf("Write of 5 succeeded\n");
-        exit(2);
+        tests_failed();
     }
     if (monitored_queue_write(buf, 4) != 4)
     {
         printf("Write of 4 failed\n");
-        exit(2);
+        tests_failed();
     }
     /* Now full. Empty a little, and refill around the end */
     if (monitored_queue_read(buf, MSG_LEN) != MSG_LEN)
     {
         printf("Read failed\n");
-        exit(2);
+        tests_failed();
     }
     if (monitored_queue_write(buf, MSG_LEN) != MSG_LEN)
     {
         printf("Write failed\n");
-        exit(2);
+        tests_failed();
     }
     /* Empty completely, checking the contents shrink correctly */
     for (;;)
@@ -374,7 +439,7 @@ static void functional_stream_tests(void)
     if (monitored_queue_read(buf, 4) != 4)
     {
         printf("Read failed\n");
-        exit(2);
+        tests_failed();
     }
     /* Nudge around the buffer */
     for (i = 1;  i < 588;  i++)
@@ -382,12 +447,12 @@ static void functional_stream_tests(void)
         if (monitored_queue_write(buf, MSG_LEN) != MSG_LEN)
         {
             printf("Write failed\n");
-            exit(2);
+            tests_failed();
         }
         if (monitored_queue_read(buf, MSG_LEN) != MSG_LEN)
         {
             printf("Read failed\n");
-            exit(2);
+            tests_failed();
         }
     }
     /* Fill the buffer, checking the contents grow correctly */
@@ -397,65 +462,67 @@ static void functional_stream_tests(void)
             break;
     }
     printf("Full at chunk %d (expected %d)\n", i, BUF_LEN/MSG_LEN + 1);
-    printf("Pointers %d %d %d\n", queue->iptr, queue->optr, queue->len);
+    if (i != BUF_LEN/MSG_LEN + 1)
+        tests_failed();
+    display_queue_pointers();
     if (monitored_queue_read(buf, MSG_LEN) != MSG_LEN)
     {
         printf("Read failed\n");
-        exit(2);
+        tests_failed();
     }
     if (monitored_queue_write(buf, MSG_LEN) != MSG_LEN)
     {
         printf("Write failed\n");
-        exit(2);
+        tests_failed();
     }
-    printf("Pointers %d %d %d\n", queue->iptr, queue->optr, queue->len);
+    display_queue_pointers();
     for (i = 1;  i < 5000;  i++)
     {
         if (monitored_queue_read(buf, MSG_LEN) != MSG_LEN)
         {
             printf("Read failed\n");
-            exit(2);
+            tests_failed();
         }
         if (monitored_queue_write(buf, MSG_LEN) != MSG_LEN)
         {
             printf("Write failed\n");
-            exit(2);
+            tests_failed();
         }
     }
-    printf("Pointers %d %d %d\n", queue->iptr, queue->optr, queue->len);
+    display_queue_pointers();
     if (monitored_queue_write(buf, 5) == 5)
     {
         printf("Write of 5 succeeded\n");
-        exit(2);
+        tests_failed();
     }
     if (monitored_queue_write(buf, 4) != 4)
     {
         printf("Write of 4 failed\n");
-        exit(2);
+        tests_failed();
     }
-    printf("Pointers %d %d %d\n", queue->iptr, queue->optr, queue->len);
+    display_queue_pointers();
     for (i = 1;  i < 5000;  i++)
     {
         if (monitored_queue_read(buf, MSG_LEN) != MSG_LEN)
         {
             printf("Read failed\n");
-            exit(2);
+            tests_failed();
         }
         if (monitored_queue_write(buf, MSG_LEN) != MSG_LEN)
         {
             printf("Write failed\n");
-            exit(2);
+            tests_failed();
         }
     }
-    printf("Pointers %d %d %d\n", queue->iptr, queue->optr, queue->len);
-    queue_delete(queue);
+    display_queue_pointers();
+    queue_free(queue);
 }
 /*- End of function --------------------------------------------------------*/
 
 static int monitored_queue_write_msg(const uint8_t buf[], int len)
 {
     int lenx;
-    
+
     lenx = queue_write_msg(queue, buf, len);
     if (lenx >= 0)
         total_in += lenx + sizeof(uint16_t);
@@ -487,10 +554,10 @@ static void functional_message_tests(void)
 
     for (i = 0;  i < MSG_LEN;  i++)
         buf[i] = i;
-    if ((queue = queue_create(BUF_LEN, QUEUE_READ_ATOMIC | QUEUE_WRITE_ATOMIC)) == NULL)
+    if ((queue = queue_init(NULL, BUF_LEN, QUEUE_READ_ATOMIC | QUEUE_WRITE_ATOMIC)) == NULL)
     {
         printf("Failed to create the queue\n");
-        exit(2);
+        tests_failed();
     }
     check_contents(total_in, total_out);
     /* Fill the buffer, checking the contents grow correctly */
@@ -500,26 +567,28 @@ static void functional_message_tests(void)
             break;
     }
     printf("Full at chunk %d (expected %d)\n", i, BUF_LEN/(MSG_LEN + sizeof(uint16_t)) + 1);
+    if (i != BUF_LEN/(MSG_LEN + sizeof(uint16_t)) + 1)
+        tests_failed();
     if ((len = monitored_queue_write_msg(buf, 5)) == 5)
     {
         printf("Write of 5 succeeded\n");
-        exit(2);
+        tests_failed();
     }
     if ((len = monitored_queue_write_msg(buf, 4)) != 4)
     {
         printf("Write of 4 failed\n");
-        exit(2);
+        tests_failed();
     }
     /* Now full. Empty a little, and refill around the end */
     if ((len = monitored_queue_read_msg(buf, MSG_LEN)) != MSG_LEN)
     {
         printf("Read failed - %d\n", len);
-        exit(2);
+        tests_failed();
     }
     if ((len = monitored_queue_write_msg(buf, MSG_LEN)) != MSG_LEN)
     {
         printf("Write failed - %d\n", len);
-        exit(2);
+        tests_failed();
     }
     /* Empty completely, checking the contents shrink correctly */
     for (;;)
@@ -530,20 +599,21 @@ static void functional_message_tests(void)
     if (len != 4)
     {
         printf("Read failed - %d\n", len);
-        exit(2);
+        tests_failed();
     }
+    /* We should now have one MSG_LEN message in the buffer */
     /* Nudge around the buffer */
     for (i = 1;  i < 527;  i++)
     {
         if ((len = monitored_queue_write_msg(buf, MSG_LEN)) != MSG_LEN)
         {
             printf("Write failed - %d\n", len);
-            exit(2);
+            tests_failed();
         }
         if ((len = monitored_queue_read_msg(buf, MSG_LEN)) != MSG_LEN)
         {
             printf("Read failed - %d\n", len);
-            exit(2);
+            tests_failed();
         }
     }
     /* Fill the buffer, checking the contents grow correctly */
@@ -552,45 +622,49 @@ static void functional_message_tests(void)
         if ((len = monitored_queue_write_msg(buf, MSG_LEN)) != MSG_LEN)
             break;
     }
-    printf("Full at chunk %d (expected %d)\n", i, BUF_LEN/(MSG_LEN + sizeof(uint16_t)) + 1);
-    printf("Pointers %d %d %d\n", queue->iptr, queue->optr, queue->len);
+    printf("Free space = %d (%d)\n", queue_free_space(queue), BUF_LEN - (total_in - total_out));
+    display_queue_pointers();
+    printf("Full at chunk %d (expected %d)\n", i, BUF_LEN/(MSG_LEN + sizeof(uint16_t)));
+    if (i != BUF_LEN/(MSG_LEN + sizeof(uint16_t)))
+        tests_failed();
+    display_queue_pointers();
 
     if ((len = monitored_queue_read_msg(buf, MSG_LEN)) != MSG_LEN)
     {
         printf("Read failed - %d\n", len);
-        exit(2);
+        tests_failed();
     }
     if ((len = monitored_queue_write_msg(buf, MSG_LEN)) != MSG_LEN)
     {
         printf("Write failed - %d\n", len);
-        exit(2);
+        tests_failed();
     }
-    printf("Pointers %d %d %d\n", queue->iptr, queue->optr, queue->len);
+    display_queue_pointers();
     for (i = 1;  i < 5000;  i++)
     {
         if ((len = monitored_queue_read_msg(buf, MSG_LEN)) != MSG_LEN)
         {
             printf("Read failed - %d\n", len);
-            exit(2);
+            tests_failed();
         }
         if ((len = monitored_queue_write_msg(buf, MSG_LEN)) != MSG_LEN)
         {
             printf("Write failed - %d\n", len);
-            exit(2);
+            tests_failed();
         }
     }
-    printf("Pointers %d %d %d\n", queue->iptr, queue->optr, queue->len);
+    display_queue_pointers();
     if ((len = monitored_queue_write_msg(buf, 5)) == 5)
     {
         printf("Write of 5 succeeded\n");
-        exit(2);
+        tests_failed();
     }
     if ((len = monitored_queue_write_msg(buf, 4)) != 4)
     {
         printf("Write of 4 failed\n");
-        exit(2);
+        tests_failed();
     }
-    printf("Pointers %d %d %d\n", queue->iptr, queue->optr, queue->len);
+    display_queue_pointers();
     for (i = 1;  i < 5000;  i++)
     {
         if (i == 527)
@@ -598,44 +672,43 @@ static void functional_message_tests(void)
             if ((len = monitored_queue_read_msg(buf, MSG_LEN)) != 4)
             {
                 printf("Read failed - %d\n", len);
-                exit(2);
+                tests_failed();
             }
         }
         if ((len = monitored_queue_read_msg(buf, MSG_LEN)) != MSG_LEN)
         {
             printf("Read failed - %d\n", len);
-            exit(2);
+            tests_failed();
         }
         if ((len = monitored_queue_write_msg(buf, MSG_LEN)) != MSG_LEN)
         {
             printf("Write failed - %d\n", len);
-            exit(2);
+            tests_failed();
         }
     }
-    printf("Pointers %d %d %d\n", queue->iptr, queue->optr, queue->len);
-    queue_delete(queue);
+    display_queue_pointers();
+    queue_free(queue);
 }
 /*- End of function --------------------------------------------------------*/
 
 int main(int argc, char *argv[])
 {
-    int i;
     int threaded_messages;
     int threaded_streams;
+    int opt;
 
     threaded_messages = FALSE;
     threaded_streams = FALSE;
-    for (i = 1;  i < argc;  i++)
+    while ((opt = getopt(argc, argv, "ms")) != -1)
     {
-        if (strcmp(argv[i], "-m") == 0)
+        switch (opt)
         {
+        case 'm':
             threaded_messages = TRUE;
-            continue;
-        }
-        if (strcmp(argv[i], "-s") == 0)
-        {
+            break;
+        case 's':
             threaded_streams = TRUE;
-            continue;
+            break;
         }
     }
 
