@@ -22,7 +22,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: t38_terminal_tests.c,v 1.30 2006/10/24 13:45:29 steveu Exp $
+ * $Id: t38_terminal_tests.c,v 1.48 2007/12/20 10:56:11 steveu Exp $
  */
 
 /*! \file */
@@ -38,27 +38,33 @@ These tests exercise the path
 #include <config.h>
 #endif
 
-#include <inttypes.h>
+#if defined(HAVE_FL_FL_H)  &&  defined(HAVE_FL_FL_CARTESIAN_H)  &&  defined(HAVE_FL_FL_AUDIO_METER_H)
+#define ENABLE_GUI
+#endif
+
 #include <stdlib.h>
 #include <stdio.h>
 #include <fcntl.h>
+#include <unistd.h>
 #include <string.h>
-#include <tgmath.h>
-#include <math.h>
 #include <assert.h>
 #include <errno.h>
 #include <sys/socket.h>
 #include <netinet/in.h>
 #include <netinet/tcp.h>
+#if !defined(__USE_MISC)
 #define __USE_MISC
+#endif
 #include <arpa/inet.h>
 #include <sys/select.h>
 #include <sys/time.h>
-#include <tiffio.h>
 
 #include "spandsp.h"
+#include "spandsp-sim.h"
 
-#include "ip_network_model.h"
+#if defined(ENABLE_GUI)
+#include "media_monitor.h"
+#endif
 
 #define SAMPLES_PER_CHUNK 160
 
@@ -68,12 +74,16 @@ These tests exercise the path
 t38_terminal_state_t t38_state_a;
 t38_terminal_state_t t38_state_b;
 
-ip_network_model_state_t *path_a_to_b;
-ip_network_model_state_t *path_b_to_a;
+g1050_state_t *path_a_to_b;
+g1050_state_t *path_b_to_a;
+
+double when = 0.0;
 
 int done[2] = {FALSE, FALSE};
 int succeeded[2] = {FALSE, FALSE};
 
+int simulate_incrementing_repeats = FALSE;
+
 static void phase_b_handler(t30_state_t *s, void *user_data, int result)
 {
     int i;
@@ -105,6 +115,8 @@ static void phase_d_handler(t30_state_t *s, void *user_data, int result)
     printf("%c: Phase D: local ident '%s'\n", i, ident);
     t30_get_far_ident(s, ident);
     printf("%c: Phase D: remote ident '%s'\n", i, ident);
+
+    printf("%c: Phase D: bits per row - min %d, max %d\n", i, s->t4.min_row_bits, s->t4.max_row_bits);
 }
 /*- End of function --------------------------------------------------------*/
 
@@ -138,12 +150,32 @@ static void phase_e_handler(t30_state_t *s, void *user_data, int result)
 static int tx_packet_handler_a(t38_core_state_t *s, void *user_data, const uint8_t *buf, int len, int count)
 {
     t38_terminal_state_t *t;
+    int i;
+    static int subst_seq = 0;
 
     /* This routine queues messages between two instances of T.38 processing */
     t = (t38_terminal_state_t *) user_data;
-    span_log(&s->logging, SPAN_LOG_FLOW, "Send seq %d, len %d, count %d\n", s->tx_seq_no, len, count);
+    if (simulate_incrementing_repeats)
+    {
+        for (i = 0;  i < count;  i++)
+        {
+            span_log(&s->logging, SPAN_LOG_FLOW, "Send seq %d, len %d\n", subst_seq, len);
 
-    ip_network_model_send(path_a_to_b, s->tx_seq_no, count, buf, len);
+            if (g1050_put(path_a_to_b, buf, len, subst_seq, when) < 0)
+                printf("Lost packet %d\n", subst_seq);
+            subst_seq = (subst_seq + 1) & 0xFFFF;
+        }
+    }
+    else
+    {
+        span_log(&s->logging, SPAN_LOG_FLOW, "Send seq %d, len %d, count %d\n", s->tx_seq_no, len, count);
+
+        for (i = 0;  i < count;  i++)
+        {
+            if (g1050_put(path_a_to_b, buf, len, s->tx_seq_no, when) < 0)
+                printf("Lost packet %d\n", s->tx_seq_no);
+        }
+    }
     return 0;
 }
 /*- End of function --------------------------------------------------------*/
@@ -151,12 +183,28 @@ static int tx_packet_handler_a(t38_core_state_t *s, void *user_data, const uint8
 static int tx_packet_handler_b(t38_core_state_t *s, void *user_data, const uint8_t *buf, int len, int count)
 {
     t38_terminal_state_t *t;
+    int i;
+    static int subst_seq = 0;
 
     /* This routine queues messages between two instances of T.38 processing */
     t = (t38_terminal_state_t *) user_data;
-    span_log(&s->logging, SPAN_LOG_FLOW, "Send seq %d, len %d, count %d\n", s->tx_seq_no, len, count);
+    if (simulate_incrementing_repeats)
+    {
+        for (i = 0;  i < count;  i++)
+        {
+            span_log(&s->logging, SPAN_LOG_FLOW, "Send seq %d, len %d\n", subst_seq, len);
 
-    ip_network_model_send(path_b_to_a, s->tx_seq_no, count, buf, len);
+            g1050_put(path_b_to_a, buf, len, subst_seq, when);
+            subst_seq = (subst_seq + 1) & 0xFFFF;
+        }
+    }
+    else
+    {
+        span_log(&s->logging, SPAN_LOG_FLOW, "Send seq %d, len %d, count %d\n", s->tx_seq_no, len, count);
+
+        for (i = 0;  i < count;  i++)
+            g1050_put(path_b_to_a, buf, len, s->tx_seq_no, when);
+    }
     return 0;
 }
 /*- End of function --------------------------------------------------------*/
@@ -166,41 +214,81 @@ int main(int argc, char *argv[])
     int msg_len;
     uint8_t msg[1024];
     int t38_version;
-    int i;
     int seq_no;
     int use_ecm;
+    int without_pacing;
+    int use_tep;
+    int model_no;
+    int speed_pattern_no;
     const char *input_file_name;
-    
+    double tx_when;
+    double rx_when;
+    int use_gui;
+    int opt;
+
     t38_version = 1;
+    without_pacing = FALSE;
+    use_tep = FALSE;
     input_file_name = INPUT_FILE_NAME;
     use_ecm = FALSE;
-    for (i = 1;  i < argc;  i++)
+    simulate_incrementing_repeats = FALSE;
+    model_no = 0;
+    speed_pattern_no = 1;
+    use_gui = FALSE;
+    while ((opt = getopt(argc, argv, "efgi:Im:ps:tv:")) != -1)
     {
-        if (strcmp(argv[i], "-e") == 0)
+        switch (opt)
         {
+        case 'e':
             use_ecm = TRUE;
-            continue;
-        }
-        if (strcmp(argv[i], "-v") == 0)
-        {
-            i++;
-            t38_version = atoi(argv[i]);
-            continue;
-        }
-        if (strcmp(argv[i], "-i") == 0)
-        {
-            i++;
-            input_file_name = argv[i];
-            continue;
+            break;
+        case 'g':
+#if defined(ENABLE_GUI)
+            use_gui = TRUE;
+#else
+            fprintf(stderr, "Graphical monitoring not available\n");
+            exit(2);
+#endif
+            break;
+        case 'i':
+            input_file_name = optarg;
+            break;
+        case 'I':
+            simulate_incrementing_repeats = TRUE;
+            break;
+        case 'm':
+            model_no = optarg[0] - 'A' + 1;
+            break;
+        case 'p':
+            without_pacing = TRUE;
+            break;
+        case 's':
+            speed_pattern_no = atoi(optarg);
+            break;
+        case 't':
+            use_tep = TRUE;
+            break;
+        case 'v':
+            t38_version = atoi(optarg);
+            break;
+        default:
+            //usage();
+            exit(2);
+            break;
         }
     }
 
-    if ((path_a_to_b = ip_network_model_init(800, 2000, 0)) == NULL)
+    printf("Using T.38 version %d\n", t38_version);
+    if (use_ecm)
+        printf("Using ECM\n");
+
+    srand48(0x1234567);
+    if ((path_a_to_b = g1050_init(model_no, speed_pattern_no, 100, 33)) == NULL)
     {
         fprintf(stderr, "Failed to start IP network path model\n");
         exit(2);
     }
-    if ((path_b_to_a = ip_network_model_init(800, 2000, 0)) == NULL)
+    if ((path_b_to_a = g1050_init(model_no, speed_pattern_no, 100, 33)) == NULL)
     {
         fprintf(stderr, "Failed to start IP network path model\n");
         exit(2);
@@ -212,6 +300,8 @@ int main(int argc, char *argv[])
         exit(2);
     }
     t38_set_t38_version(&t38_state_a.t38, t38_version);
+    t38_terminal_set_config(&t38_state_a, without_pacing);
+    t38_terminal_set_tep_mode(&t38_state_a, use_tep);
     span_log_set_level(&t38_state_a.logging, SPAN_LOG_DEBUG | SPAN_LOG_SHOW_TAG | SPAN_LOG_SHOW_SAMPLE_TIME);
     span_log_set_tag(&t38_state_a.logging, "T.38-A");
     span_log_set_level(&t38_state_a.t38.logging, SPAN_LOG_DEBUG | SPAN_LOG_SHOW_TAG | SPAN_LOG_SHOW_SAMPLE_TIME);
@@ -220,6 +310,7 @@ int main(int argc, char *argv[])
     span_log_set_tag(&t38_state_a.t30_state.logging, "T.38-A");
 
     t30_set_local_ident(&t38_state_a.t30_state, "11111111");
+    t30_set_local_nsf(&t38_state_a.t30_state, (const uint8_t *) "\x50\x00\x00\x00Spandsp\x00", 12);
     t30_set_tx_file(&t38_state_a.t30_state, input_file_name, -1, -1);
     t30_set_phase_b_handler(&t38_state_a.t30_state, phase_b_handler, (void *) (intptr_t) 'A');
     t30_set_phase_d_handler(&t38_state_a.t30_state, phase_d_handler, (void *) (intptr_t) 'A');
@@ -234,6 +325,8 @@ int main(int argc, char *argv[])
         exit(2);
     }
     t38_set_t38_version(&t38_state_b.t38, t38_version);
+    t38_terminal_set_config(&t38_state_b, without_pacing);
+    t38_terminal_set_tep_mode(&t38_state_b, use_tep);
     span_log_set_level(&t38_state_b.logging, SPAN_LOG_DEBUG | SPAN_LOG_SHOW_TAG | SPAN_LOG_SHOW_SAMPLE_TIME);
     span_log_set_tag(&t38_state_b.logging, "T.38-B");
     span_log_set_level(&t38_state_b.t38.logging, SPAN_LOG_DEBUG | SPAN_LOG_SHOW_TAG | SPAN_LOG_SHOW_SAMPLE_TIME);
@@ -242,6 +335,7 @@ int main(int argc, char *argv[])
     span_log_set_tag(&t38_state_b.t30_state.logging, "T.38-B");
 
     t30_set_local_ident(&t38_state_b.t30_state, "22222222");
+    t30_set_local_nsf(&t38_state_b.t30_state, (const uint8_t *) "\x50\x00\x00\x00Spandsp\x00", 12);
     t30_set_rx_file(&t38_state_b.t30_state, OUTPUT_FILE_NAME, -1);
     t30_set_phase_b_handler(&t38_state_b.t30_state, phase_b_handler, (void *) (intptr_t) 'B');
     t30_set_phase_d_handler(&t38_state_b.t30_state, phase_d_handler, (void *) (intptr_t) 'B');
@@ -250,6 +344,10 @@ int main(int argc, char *argv[])
     if (use_ecm)
         t30_set_supported_compressions(&t38_state_b.t30_state, T30_SUPPORT_T4_1D_COMPRESSION | T30_SUPPORT_T4_2D_COMPRESSION | T30_SUPPORT_T6_COMPRESSION);
 
+#if defined(ENABLE_GUI)
+    if (use_gui)
+        start_media_monitor();
+#endif
     for (;;)
     {
         span_log_bump_samples(&t38_state_a.logging, SAMPLES_PER_CHUNK);
@@ -262,14 +360,33 @@ int main(int argc, char *argv[])
         done[0] = t38_terminal_send_timeout(&t38_state_a, SAMPLES_PER_CHUNK);
         done[1] = t38_terminal_send_timeout(&t38_state_b, SAMPLES_PER_CHUNK);
 
-        while ((msg_len = ip_network_model_get(path_a_to_b, SAMPLES_PER_CHUNK, msg, 1024, &seq_no)) >= 0)
-            t38_core_rx_ifp_packet(&t38_state_b.t38, seq_no, msg, msg_len);
-        while ((msg_len = ip_network_model_get(path_b_to_a, SAMPLES_PER_CHUNK, msg, 1024, &seq_no)) >= 0)
-            t38_core_rx_ifp_packet(&t38_state_a.t38, seq_no, msg, msg_len);
+        when += 0.02;
 
+        while ((msg_len = g1050_get(path_a_to_b, msg, 1024, when, &seq_no, &tx_when, &rx_when)) >= 0)
+        {
+#if defined(ENABLE_GUI)
+            if (use_gui)
+                media_monitor_rx(seq_no, tx_when, rx_when);
+#endif
+            t38_core_rx_ifp_packet(&t38_state_b.t38, msg, msg_len, seq_no);
+        }
+        while ((msg_len = g1050_get(path_b_to_a, msg, 1024, when, &seq_no, &tx_when, &rx_when)) >= 0)
+        {
+#if defined(ENABLE_GUI)
+            if (use_gui)
+                media_monitor_rx(seq_no, tx_when, rx_when);
+#endif
+            t38_core_rx_ifp_packet(&t38_state_a.t38, msg, msg_len, seq_no);
+        }
         if (done[0]  &&  done[1])
             break;
+#if defined(ENABLE_GUI)
+        if (use_gui)
+            media_monitor_update_display();
+#endif
     }
+    t38_terminal_release(&t38_state_a);
+    t38_terminal_release(&t38_state_b);
     if (!succeeded[0]  ||  !succeeded[1])
     {
         printf("Tests failed\n");
