@@ -22,21 +22,22 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
  *
- * $Id: v8_tests.c,v 1.17 2006/10/24 13:45:30 steveu Exp $
+ * $Id: v8_tests.c,v 1.20 2007/11/10 11:14:59 steveu Exp $
  */
 
 /*! \page v8_tests_page V.8 tests
 \section v8_tests_page_sec_1 What does it do?
 */
 
+#ifdef HAVE_CONFIG_H
+#include "config.h"
+#endif
+
 #include <stdlib.h>
-#include <inttypes.h>
 #include <stdio.h>
-#include <math.h>
 #include <fcntl.h>
 #include <string.h>
 #include <audiofile.h>
-#include <tiffio.h>
 
 #include "spandsp.h"
 
